#include <qpe/config.h>
#include <qapplication.h>
#include <qpe/qcopenvelope_qws.h>

#include <qheader.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qpe/mimetype.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qiconset.h>

#include "AplDialog.h"

AplDialog::AplDialog(QWidget* parent,  const char* name, bool modal, WFlags fl )    : QDialog( parent, name, modal, fl )
{
	setCaption(tr("Select Application"));
	layout = new QVBoxLayout(this);
	setTab = new QTabWidget( this);
	layout->addWidget(setTab);

	desktop = "";

	set = new AppLnkSet(MimeType::appsFolderName());
	menuSet.setAutoDelete(TRUE);
	loadMenuTab();
	loadDesktop();

	showMaximized();
}

AplDialog::~AplDialog() 
{
	delete set;
}


void AplDialog::loadMenuTab()
{
	menuSet.clear();
	QStringList types = set->types();
	for ( QStringList::Iterator it=types.begin(); it!=types.end(); ++it) {
		if ( !(*it).isEmpty() ) {
			makeMenuTab(QIconSet(set->typePixmap(*it)),set->typeName(*it),*it);
		}
	}
	makeMenuTab(QIconSet(),tr("Others"),"others");
}

void AplDialog::makeMenuTab(const QIconSet& ic,const QString& s ,const QString& type)
{
	QWidget* t = new QWidget(setTab);
	QBoxLayout* b = new QVBoxLayout( t );
	QListView* l = new QListView(t);
	l->addColumn( tr( "Application Name" ) ,-1);
	l->addColumn(" ",20);
	l->addColumn( tr( "desktop file " ) ,-1);
	b->addWidget(l);
	setTab->insertTab(t,ic,s);
	MenuSet* m = new MenuSet();
	connect (l,SIGNAL(selectionChanged(QListViewItem* )), this, SLOT(selectChanged(QListViewItem* ) ) );

	m->type = type;
	m->wid = t;
	m->lay = b;
	m->list = l;
	menuSet.append(m);
}

void AplDialog::loadDesktop()
{
	QStringList apl;
	{
		Config cf= Config("Launcher");
		cf.setGroup("Preload");
		apl = cf.readListEntry("Apps",',');
	}
	QList<AppLnk> aplist = set->children();
	for (QListIterator<AppLnk> it(aplist) ; it.current() ; ++it) {
		if (it.current()->type() != "Separator") {
			MenuSet* m = findTab(it.current()->type());
			QListViewItem *item = new QListViewItem(m->list);
			item->setText(AplName,it.current()->name());
			item->setPixmap(AplName,it.current()->pixmap());
			item->setText(DesktopFile,it.current()->linkFile());
		}
	}
}

MenuSet* AplDialog::findTab(QString s)
{
	for (QListIterator<MenuSet> it(menuSet) ; it.current() ; ++it) {
		if ((*it)->type == s) {
			return it;
		}
	}
	return menuSet.getLast();
}

QString AplDialog::desktopName()
{
	return desktop;
}

void AplDialog::accept () 
{
	QDialog::accept();
}
void AplDialog::selectChanged(QListViewItem* i)
{
	desktop = i->text(DesktopFile);
}
