#include <qpe/config.h>
#include <qapplication.h>

#include <qcheckbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qmessagebox.h>
#include <qstring.h>

#include <string.h>

#include "FLUtil.h"
#include "MenuItemDialog.h"
#include "MListItem.h"
#include "../FileLaunch/FLib.h"
#include "../FileLaunch/FileLaunch.h"
#include "AplDialog.h"

FLUtil::FLUtil() : QMainWindow()
{
	setCaption(tr("FLUtil")) ;
	setToolBarsMovable(false) ;
	memset(keyMapping,0,sizeof(keyMapping));

    tabWidget = new QTabWidget( this, "tabWidget" );

	makeConfTab();
	makeMenuTab();
	makeOptTab();
	makeColorTab();

	tabWidget->setCurrentPage(1);

	setCentralWidget( tabWidget );

	loadCfg();

	setColorTab(0);

	connect( confList,  SIGNAL( doubleClicked(QListViewItem*)),this,SLOT( listSelected(QListViewItem*)) );
	connect( confList,  SIGNAL( returnPressed(QListViewItem*)),this,SLOT( listSelected(QListViewItem*)) );

	connect( addButton, SIGNAL( clicked() ), this, SLOT( addClicked() ) );
	connect( modButton, SIGNAL( clicked() ), this, SLOT( modClicked() ) );
	connect( delButton, SIGNAL( clicked() ), this, SLOT( delClicked() ) );
	connect( upButton,  SIGNAL( clicked() ), this, SLOT( upClicked() ) );
	connect( downButton,SIGNAL( clicked() ), this, SLOT( downClicked() ) );
	connect( levelButton,SIGNAL( clicked() ), this, SLOT( levelClicked() ) );
	connect( menuList,  SIGNAL( doubleClicked(QListViewItem*)),this,SLOT( listSelected(QListViewItem*)) );
	connect( menuList,  SIGNAL( returnPressed(QListViewItem*)),this,SLOT( listSelected(QListViewItem*)) );
	connect( memoAlterButton, SIGNAL( clicked()) , this ,SLOT( getMemoAlter()) );
	connect( optOkButton, SIGNAL( clicked()) , this ,SLOT( optOkClicked()) );
	connect( optCancelButton, SIGNAL( clicked()) , this ,SLOT( optCancelClicked()) );
	
	connect( nlRedBox  , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( nlGreenBox, SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( nlBlueBox , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( clRedBox  , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( clGreenBox, SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( clBlueBox , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( nbRedBox  , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( nbGreenBox, SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( nbBlueBox , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( sbRedBox  , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( sbGreenBox, SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( sbBlueBox , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( cbRedBox  , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( cbGreenBox, SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );
	connect( cbBlueBox , SIGNAL(valueChanged(int)) , this ,SLOT( setColorTab(int)) );

	connect( colorOkButton, SIGNAL( clicked()) , this ,SLOT( setColor()) );
	connect( colorCancelButton, SIGNAL( clicked()) , this ,SLOT( reloadColor()) );

}

FLUtil::~FLUtil() 
{
}
void FLUtil::closeEvent(QCloseEvent *e)
{
	saveCfg();
	e->accept();
}

void FLUtil::saveCfg()
{
	Config cf = Config("FileLaunch");
	
	cf.setGroup("config");
	cf.writeEntry("ShellExtension",shellExt.join(";"));
	cf.writeEntry("TextExtension",textExt.join(";"));
	cf.writeEntry("MemoAlter",memoAlter);
	cf.writeEntry("ViewFontSize",viewFontSize);
	cf.writeEntry("FileFontSize",fileFontSize);
	cf.writeEntry("UseSudo",useSudo);
	cf.writeEntry("SetSymmetry",symmetry);
	cf.writeEntry("MarkAuto",markAuto);

	cf.setGroup("color");
	cf.writeEntry("UseCustomBack",customBack);
	cf.writeEntry("NormalLinkR",nLink.red());
	cf.writeEntry("NormalLinkG",nLink.green());
	cf.writeEntry("NormalLinkB",nLink.blue());
	cf.writeEntry("CardLinkR",cLink.red());
	cf.writeEntry("CardLinkG",cLink.green());
	cf.writeEntry("CardLinkB",cLink.blue());
	cf.writeEntry("NormalBackR",nBack.red());
	cf.writeEntry("NormalBackG",nBack.green());
	cf.writeEntry("NormalBackB",nBack.blue());
	cf.writeEntry("SdBackR",sBack.red());
	cf.writeEntry("SdBackG",sBack.green());
	cf.writeEntry("SdBackB",sBack.blue());
	cf.writeEntry("CfBackR",cBack.red());
	cf.writeEntry("CfBackG",cBack.green());
	cf.writeEntry("CfBackB",cBack.blue());

	cf.setGroup("menu");
	cf.clearGroup();
	int no = 0;

	for (QListViewItemIterator it(confList) ; it.current() ; ++it) {
		cf.writeEntry("Item_" + FLib::fixNum(no++,3 ),toCfg((MListItem*)(it.current())));
	}
	for (QListViewItemIterator it(menuList) ; it.current() ; ++it) {
		cf.writeEntry("Item_" + FLib::fixNum(no++,3 ),toCfg((MListItem*)(it.current())));
	}
}
QString FLUtil::toCfg(MListItem* m)
{
	QString o = QString::number(m->depth()) + ",";
	o += QString::number(m->type()) + ",";
	o += QString::number(m->autoT()) + ",";
	o += m->name() + "," + m->pathName() + "," + m->paramForm() + ",";
	o += QString::number(m->p1Type()) + "," + QString::number(m->p2Type()) + "," + m->msgDisp() + ",";
	o += QString::number(m->shortKey()) + "," + QString::number(m->hotKey());
	return o;
}

void FLUtil::addClicked()
{
	MListItem* m = (MListItem*) menuList->currentItem();
	if (m && (m->type() == MenuItem::Menu)) {
		m = new MListItem(m,m->firstChild());
	} else if (m && (m->depth()>0)) {
		m = new MListItem(m->parent(),m);
	} else {
		 m = new MListItem(menuList,m);
	}
	setKeyMap(m);
	MenuItemDialog d(m,keyMapping,false,this);
	d.showMaximized();
	if (d.exec() == QDialog::Rejected) {
		delete m;
	} else {
		loadPixmap(m);
	}
}
void FLUtil::modClicked()
{
	MListItem* m = (MListItem*) menuList->currentItem();
	if (m ) {
		setKeyMap(m);
		MenuItemDialog d(m,keyMapping,false,this);
		d.showMaximized();
		if (d.exec() == QDialog::Accepted) {
			loadPixmap(m);
		}
	}
}
void FLUtil::delClicked()
{
	MListItem* m = (MListItem*) menuList->currentItem();
	if (m ) {
		if (QMessageBox::warning(this, tr("FLUtil"),tr("item ") + m->name() + tr(" delete ok?"),
			tr("Yes"),tr("No"),QString::null,1) == 0 ) {
				delete m;
		}
	}
}
void FLUtil::upClicked()
{
	MListItem* m =(MListItem*) menuList->currentItem();
	if (m != NULL && m != menuList->firstChild()) {
		MListItem* a = (MListItem*)m->itemAbove();
		if (m->depth() == a->depth()) {
			a->moveItem(m);
		} else if (m->depth() < a->depth() ) {
			a = (MListItem*) a->parent();
			if (QMessageBox::warning(this, tr("FLUtil"),
				tr("go to under ") + a->name() + tr(" menu. ok?"),
				tr("Yes"),tr("No"),QString::null,1) == 0 ) {
				if (m->depth() == 0) {
					menuList->takeItem(m);
				} else {
					m->parent()->takeItem(m);
				}
				a->insertItem(m);
			} else {
				while (m->depth() < a->depth()) {
					a = (MListItem*) a->parent();
				}
				a->moveItem(m);
			}
		} else if (m->depth() > a->depth() ) {
			a = (MListItem*) a->parent();
			QString n;
			if (! a) {
				n = tr("MainMenu");
			} else {
				n = a->name();
			}
			if (QMessageBox::warning(this, tr("FLUtil"),
				tr("go to under ") + n + tr(" menu. ok?"),
				tr("Yes"),tr("No"),QString::null,1) == 0 ) {
				m->parent()->takeItem(m);
				if (! a) {
					menuList->insertItem(m);
				} else {
					a->insertItem(m);
				}
			}
		}
	}
}
void FLUtil::downClicked()
{
	MListItem* m =(MListItem*) menuList->currentItem();
	if (m != NULL && m->itemBelow()!= NULL ) {
		MListItem* a = (MListItem*)m->itemBelow();
		if (a->type() == MenuItem::Menu) {
			if (QMessageBox::warning(this, tr("FLUtil"),
				tr("go to under ") + a->name() + tr(" menu. ok?"),
				tr("Yes"),tr("No"),QString::null,1) == 0 ) {
				if (m->depth() == 0) {
					menuList->takeItem(m);
				} else {
					m->parent()->takeItem(m);
				}
				a->insertItem(m);
				return;
			}
		}
		if (m->depth() <= a->depth()) {
			if (m->depth() > a->depth()) {
				do {
					a = (MListItem*) a->parent();
				} while (m->depth() > a->depth());
			}
			m->moveItem(a);
		}
	}
/*
	if (m != NULL & m->itemBelow() != NULL && m->depth() <= m->itemBelow()->depth()) {
		MListItem* a = (MListItem*)m->itemBelow();
		if (m->depth() > a->depth()) {
			do {
				a = (MListItem*) a->parent();
			} while (m->depth() > a->depth());
		}
		m->moveItem(a);
	}
*/
}

void FLUtil::levelClicked()
{
	for (QListViewItemIterator it(menuList) ; it.current() ; ++it) {
		it.current()->setOpen(TRUE);
	}
}

void FLUtil::listSelected(QListViewItem* item)
{
	setKeyMap(item);
	MenuItemDialog d((MListItem*) item,keyMapping, (item->listView() == confList) ,this);
	d.showMaximized();
	if (d.exec() == QDialog::Accepted) {
		loadPixmap((MListItem*) item);
	}
}

void FLUtil::setKeyMap(QListViewItem* item)
{
	memset(keyMapping,0,sizeof(keyMapping));
	for (QListViewItemIterator it(confList) ; it.current() ; ++it) {
		MListItem* m = (MListItem*)it.current();
		if (it.current() != item && m->hotKey() > 0) {
			keyMapping[m->hotKey()] = 1;
		}
	}
	for (QListViewItemIterator it(menuList) ; it.current() ; ++it) {
		MListItem* m = (MListItem*)it.current();
		if (it.current() != item && m->hotKey() > 0) {
			keyMapping[m->hotKey()] = 1;
		}
	}
}

void FLUtil::loadCfg()
{
	Config cf = Config("FileLaunch");
	cf.setGroup("config");
	shellExt = cf.readListEntry("ShellExtension",';');
	textExt = cf.readListEntry("TextExtension",';');
	memoAlter = cf.readEntry("MemoAlter");
	fileFontSize = cf.readNumEntry("FileFontSize",18);
	viewFontSize = cf.readNumEntry("ViewFontSize",18);
	useSudo = cf.readBoolEntry("UseSudo",false);
	symmetry = cf.readBoolEntry("SetSymmetry",false);
	markAuto = cf.readBoolEntry("MarkAuto",true);

	int r;
	int g;
	int b;
	cf.setGroup("color");
	customBack = cf.readBoolEntry("UseCustomBack",false);
	customBox->setChecked(customBack);
	r = cf.readNumEntry("NormalLinkR",0);
	g = cf.readNumEntry("NormalLinkG",0);
	b = cf.readNumEntry("NormalLinkB",255);
	nLink.setRgb(r,g,b);
	r = cf.readNumEntry("CardLinkR",0);
	g = cf.readNumEntry("CardLinkG",0);
	b = cf.readNumEntry("CardLinkB",255);
	cLink.setRgb(r,g,b);
	r = cf.readNumEntry("NormalBackR",255);
	g = cf.readNumEntry("NormalBackG",255);
	b = cf.readNumEntry("NormalBackB",255);
	nBack.setRgb(r,g,b);
	r = cf.readNumEntry("SdBackR",200);
	g = cf.readNumEntry("SdBackG",255);
	b = cf.readNumEntry("SdBackB",255);
	sBack.setRgb(r,g,b);
	r = cf.readNumEntry("CfBackR",255);
	g = cf.readNumEntry("CfBackG",255);
	b = cf.readNumEntry("CfBackB",200);
	cBack.setRgb(r,g,b);
	reloadColor();

	loadOpt();

	cf.setGroup("menu");
	QStringList li;
	MListItem* m = NULL;
	for (int no = 0 ; cf.hasKey("Item_" + FLib::fixNum(no,3 )) ; ++no) {
		li = cf.readListEntry("Item_" + FLib::fixNum(no,3 ),',');
		if (no < MenuItem::reserveItem ) {
			m = new MListItem(confList,m);
		} else if (li[0].toInt() > 0 ) {
			if (m->depth()+1 == li[0].toInt()) {
				m = new MListItem(m,m->firstChild());
			} else {
				m = new MListItem(m->parent(),m);
			}
		} else {
			while (m!= NULL && m->depth() >0 ) {
				m = (MListItem*) m->parent();
			}
			m = new MListItem(menuList,m);
		}
		m->setItems(
			li[1].toInt(),li[2].toInt(),
			li[3],li[4],li[5],
			li[6].toInt(),li[7].toInt(),li[8], //li[9].toInt(),
			li[9].toInt(),li[10].toInt()
		);
		loadPixmap(m);
	}
}

void FLUtil::loadPixmap(MListItem* m)
{
	if (m->type() == MenuItem::QtApps) {
		const AppLnk* app = new AppLnk(m->pathName());
		m->setPixmap(0,app->pixmap());
		delete app;
	} else if (m->type() != MenuItem::Menu) {
		m->setPixmap(0,Resource::loadPixmap( "konsole" ));
	} else {
		m->setPixmap(0,Resource::loadPixmap( "folder" ));
	}
}

void FLUtil::makeConfTab()
{
// config tab

    conftab = new QWidget( tabWidget);
    tabLayout = new QGridLayout( conftab ,ConfRowMax,ConfColMax); 
    tabLayout->setSpacing( 6 );
    tabLayout->setMargin( 11 );

	confLabel = new QLabel(conftab);
	confLabel->setText( tr("config for System Menu") );
	tabLayout->addMultiCellWidget( confLabel, confLabelRow, confLabelRow, confMinCol, confMaxCol );

	confList = new QListView(conftab);
	confList->addColumn( tr( "Menu Title Name" ) ,-1);
	confList->setSorting(-1);
	tabLayout->addMultiCellWidget( confList, confListRow, confListRow, confMinCol, confMaxCol );

	tabWidget->insertTab( conftab, tr( "CONFIG" ) );
}

void FLUtil::makeMenuTab()
{
// menu tab

	menuTab = new QWidget( tabWidget);
	menuTabLayout = new QGridLayout( menuTab ,MenuRowMax,MenuColMax);
	menuTabLayout->setSpacing( 6 );
	menuTabLayout->setMargin( 11 );

	addButton = new QPushButton( menuTab );
	addButton->setText( tr( "ADD" ) );
	addButton->setAccel(CTRL+Key_A);
	menuTabLayout->addWidget( addButton, buttonRow, addButtonCol );

	modButton = new QPushButton( menuTab );
	modButton->setText( tr( "MOD" ) );
	menuTabLayout->addWidget( modButton, buttonRow, modButtonCol );

	delButton = new QPushButton( menuTab );
	delButton->setText( tr( "DEL" ) );
	delButton->setAccel(Key_Delete);
	menuTabLayout->addWidget( delButton, buttonRow, delButtonCol );

	upButton = new QPushButton( menuTab );
	upButton->setText( tr( "UP" ) );
	upButton->setAccel(CTRL+Key_U);
	menuTabLayout->addWidget( upButton, buttonRow, upButtonCol );

	downButton = new QPushButton( menuTab );
	downButton->setText( tr( "DOWN" ) );
	downButton->setAccel(CTRL+Key_D);
	menuTabLayout->addWidget( downButton, buttonRow, downButtonCol );

	levelButton = new QPushButton( menuTab );
	levelButton->setText( tr( "LEVEL" ) );
	levelButton->setAccel(CTRL+Key_L);
	menuTabLayout->addWidget( levelButton, buttonRow, levelButtonCol );

	menuList = new QListView( menuTab, "menuList" );
	menuList->addColumn( tr( "Menu Name" ) ,-1);
	menuList->addColumn( tr( "Type" ) );
	menuList->setRootIsDecorated(TRUE);
	menuList->setAllColumnsShowFocus(TRUE);
	menuList->setSorting(-1);
	menuTabLayout->addMultiCellWidget( menuList, listRow, listRow, addButtonCol, levelButtonCol );

	tabWidget->insertTab( menuTab, tr( "MENU" ) );

}

void FLUtil::makeOptTab()
{
	optTab = new QWidget( tabWidget);
	tabWidget->insertTab( optTab, tr( "OPTION" ) );
	optTabLayout = new QGridLayout( optTab,OptRowMax, OptColMax);
	optTabLayout->setSpacing( 2 );
	optTabLayout->setMargin( 2 );
	
	fileFontLabel = new QLabel( optTab );
	fileFontLabel->setText(tr("FileFontSize"));
	optTabLayout->addMultiCellWidget( fileFontLabel, FileFontRow, FileFontRow, LeftCol, MidCol );
	fileFontBox = new QComboBox( optTab );
	fileFontBox->insertItem("10");
	fileFontBox->insertItem("12");
	fileFontBox->insertItem("16");
	fileFontBox->insertItem("18");
	fileFontBox->insertItem("20");
	optTabLayout->addMultiCellWidget( fileFontBox, FileFontRow, FileFontRow, RightCol, RightCol );

	viewFontLabel = new QLabel( optTab );
	viewFontLabel->setText(tr("ViewFontSize"));
	optTabLayout->addMultiCellWidget( viewFontLabel, ViewFontRow, ViewFontRow, LeftCol, MidCol );
	viewFontBox = new QComboBox( optTab );
	viewFontBox->insertItem("10");
	viewFontBox->insertItem("12");
	viewFontBox->insertItem("16");
	viewFontBox->insertItem("18");
	viewFontBox->insertItem("20");
	optTabLayout->addMultiCellWidget( viewFontBox, ViewFontRow, ViewFontRow, RightCol, RightCol );

	memoAlterLabel = new QLabel( optTab );
	memoAlterLabel->setText(tr("MemoAlterApl"));
	optTabLayout->addMultiCellWidget( memoAlterLabel, MemoAlterRow, MemoAlterRow, LeftCol, LeftCol );
	memoAlterEdit = new QLineEdit( optTab);
	optTabLayout->addMultiCellWidget( memoAlterEdit, MemoAlterRow, MemoAlterRow, MidCol, MidCol );
	memoAlterButton = new QPushButton( optTab);
	memoAlterButton->setText(tr("Select"));
	optTabLayout->addMultiCellWidget( memoAlterButton, MemoAlterRow, MemoAlterRow, RightCol, RightCol );

	textExtLabel = new QLabel( optTab);
	textExtLabel->setText(tr("Text Extension"));
	optTabLayout->addMultiCellWidget( textExtLabel, TextExtRow, TextExtRow, LeftCol, LeftCol );
	textExtEdit = new QLineEdit( optTab);
	optTabLayout->addMultiCellWidget( textExtEdit, TextExtRow, TextExtRow, MidCol, RightCol );

	shellExtLabel = new QLabel( optTab);
	shellExtLabel->setText(tr("Shell Extension"));
	optTabLayout->addMultiCellWidget( shellExtLabel, ShellExtRow, ShellExtRow, LeftCol, LeftCol );
	shellExtEdit = new QLineEdit( optTab);
	optTabLayout->addMultiCellWidget( shellExtEdit, ShellExtRow, ShellExtRow, MidCol, RightCol );

	sudoCheckBox = new QCheckBox( optTab);
	sudoCheckBox->setText(tr("use sudo"));
	optTabLayout->addMultiCellWidget( sudoCheckBox, SudoRow, SudoRow, LeftCol, RightCol );
	
	symmetryBox = new QCheckBox( optTab);
	symmetryBox->setText(tr("file view symmetrically"));
	optTabLayout->addMultiCellWidget( symmetryBox, SymmetryRow, SymmetryRow, LeftCol, RightCol );

	markAutoBox = new QCheckBox( optTab);
	markAutoBox->setText(tr("use AutoMark"));
	optTabLayout->addMultiCellWidget( markAutoBox, MarkAutoRow, MarkAutoRow, LeftCol, RightCol );

	optOkButton = new QPushButton( optTab);
	optOkButton->setText(tr("OK"));
	optOkButton->setAccel(CTRL+Key_S);
	optTabLayout->addMultiCellWidget( optOkButton, OkRow, OkRow, LeftCol, LeftCol );
	
	optCancelButton = new QPushButton(optTab);
	optCancelButton->setText(tr("Cancel"));
	optCancelButton->setAccel(CTRL+Key_R);
	optTabLayout->addMultiCellWidget( optCancelButton, OkRow, OkRow, RightCol, RightCol );
	
	sizeToIndex.insert(10,0);
	sizeToIndex.insert(12,1);
	sizeToIndex.insert(16,2);
	sizeToIndex.insert(18,3);
	sizeToIndex.insert(20,4);
}

void FLUtil::getMemoAlter()
{
	AplDialog d(this);
	if (d.exec() == QDialog::Accepted) {
		memoAlterEdit->setText(d.desktopName());
	}
}

void FLUtil::loadOpt()
{
	shellExtEdit->setText(shellExt.join(";"));
	textExtEdit->setText(textExt.join(";"));
	memoAlterEdit->setText(memoAlter);
	sudoCheckBox->setChecked(useSudo);
	symmetryBox->setChecked(symmetry);
	markAutoBox->setChecked(markAuto);
	fileFontBox->setCurrentItem(sizeToIndex[fileFontSize]);
	viewFontBox->setCurrentItem(sizeToIndex[viewFontSize]);
}

void FLUtil::setOpt()
{
	shellExt = QStringList::split(";",shellExtEdit->text());
	textExt = QStringList::split(";",textExtEdit->text());
	memoAlter = memoAlterEdit->text();
	useSudo = sudoCheckBox->isChecked();
	symmetry = symmetryBox->isChecked();
	markAuto = markAutoBox->isChecked();
	fileFontSize = indexToSize[fileFontBox->currentItem()];
	viewFontSize = indexToSize[viewFontBox->currentItem()];

	setSudo(FileLaunch::CopyMenu,useSudo);
	setSudo(FileLaunch::MoveMenu,useSudo);
	setSudo(FileLaunch::RenameMenu,useSudo);
	setSudo(FileLaunch::ExtMenu,useSudo);
}
void FLUtil::setSudo(int i, bool b)
{
	int j = 0;
	for (QListViewItemIterator it(confList) ; it.current() ; ++it) {
		if (j == i) {
			MListItem* m = (MListItem*) it.current();
			QString p = m->paramForm();
			p.replace(QRegExp("^sudo "),"");
			if (b) {	// set
				p = "sudo " + p;
			}
			m->setParam(p);
			break;
		}
		++j;
	}
}

void FLUtil::optOkClicked()
{
	setOpt();
	loadOpt();
}
void FLUtil::optCancelClicked()
{
	loadOpt();
}

void FLUtil::makeColorTab()
{
	colorTab = new QWidget( tabWidget);
	tabWidget->insertTab( colorTab, tr( "COLOR" ) );
	colorTabLayout = new QGridLayout( colorTab,ColorRowMax, ColorColMax);
	colorTabLayout->setSpacing( 2 );
	colorTabLayout->setMargin( 2 );
	
	redLabel = new QLabel( colorTab );
	redLabel->setText("<font color=\"#FF0000\">" + tr("Red")+ "</font>");
	colorTabLayout->addWidget( redLabel, LabelRow, RedCol );
	greenLabel = new QLabel( colorTab );
	greenLabel->setText("<font color=\"#00FF00\">" + tr("Green")+ "</font>");
	colorTabLayout->addWidget( greenLabel, LabelRow, GreenCol );
	blueLabel = new QLabel( colorTab );
	blueLabel->setText("<font color=\"#0000FF\">" + tr("Blue")+ "</font>");
	colorTabLayout->addWidget( blueLabel, LabelRow, BlueCol );
	
	normalLinkLabel = new QLabel( colorTab );
	colorTabLayout->addWidget( normalLinkLabel, NormalLinkRow, LabelCol );
	nlRedBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( nlRedBox, NormalLinkRow, RedCol );
	nlGreenBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( nlGreenBox, NormalLinkRow, GreenCol );
	nlBlueBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( nlBlueBox, NormalLinkRow, BlueCol );

	cardLinkLabel = new QLabel( colorTab );
	colorTabLayout->addWidget( cardLinkLabel, CardLinkRow, LabelCol );
	clRedBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( clRedBox, CardLinkRow, RedCol );
	clGreenBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( clGreenBox, CardLinkRow, GreenCol );
	clBlueBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( clBlueBox, CardLinkRow, BlueCol );

	customBox = new QCheckBox(colorTab);
	customBox->setText(tr("Use custom Backcolor"));
	colorTabLayout->addMultiCellWidget( customBox, CustomRow, CustomRow, LabelCol, BlueCol );

	normalBackLabel = new QLabel( colorTab );
	colorTabLayout->addWidget( normalBackLabel, NormalBackRow, LabelCol );
	nbRedBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( nbRedBox, NormalBackRow, RedCol );
	nbGreenBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( nbGreenBox, NormalBackRow, GreenCol );
	nbBlueBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( nbBlueBox, NormalBackRow, BlueCol );

	sdBackLabel = new QLabel( colorTab );
	colorTabLayout->addWidget( sdBackLabel, SdBackRow, LabelCol );
	sbRedBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( sbRedBox, SdBackRow, RedCol );
	sbGreenBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( sbGreenBox, SdBackRow, GreenCol );
	sbBlueBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( sbBlueBox, SdBackRow, BlueCol );

	cfBackLabel = new QLabel( colorTab );
	colorTabLayout->addWidget( cfBackLabel, CfBackRow, LabelCol );
	cbRedBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( cbRedBox, CfBackRow, RedCol );
	cbGreenBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( cbGreenBox, CfBackRow, GreenCol );
	cbBlueBox = new QSpinBox( 0,255,1,colorTab );
	colorTabLayout->addWidget( cbBlueBox, CfBackRow, BlueCol );

	colorOkButton = new QPushButton( colorTab);
	colorOkButton->setText(tr("OK"));
	colorOkButton->setAccel(CTRL+Key_S);
	colorTabLayout->addWidget( colorOkButton, ColorButtonRow, LabelCol);
	
	colorCancelButton = new QPushButton(colorTab);
	colorCancelButton->setText(tr("Cancel"));
	colorCancelButton->setAccel(CTRL+Key_R);
	colorTabLayout->addWidget( colorCancelButton, ColorButtonRow,BlueCol );
	
	for (int i = 1 ; i < ColorRowMax ; ++ i ) {
		colorTabLayout->setRowStretch(i,1);
	}
	
}

void FLUtil::setColorTab(int i)
{
	QString color;
	color.sprintf("<font color =\"#%02X%02X%02X",nlRedBox->value(),nlGreenBox->value(),nlBlueBox->value());
	color +=  "\">" + tr("Normal SymLink") + "</font>";
	normalLinkLabel->setText(color);

	color.sprintf("<font color =\"#%02X%02X%02X",clRedBox->value(),clGreenBox->value(),clBlueBox->value());
	color +=  "\">" + tr("SymLink to Card") + "</font>";
	cardLinkLabel->setText(color);

	color.sprintf("<table border = 1 bgcolor =\"#%02X%02X%02X",nbRedBox->value(),nbGreenBox->value(),nbBlueBox->value());
	color +=  "\"><tr><td>" + tr("Normal BackColor") + "</td></tr></table>";
	normalBackLabel->setText(color);
	color.sprintf("<table border = 1 bgcolor =\"#%02X%02X%02X",sbRedBox->value(),sbGreenBox->value(),sbBlueBox->value());
	color +=  "\"><tr><td>" + tr("BackColor of SD") + "</td></tr></table>";
	sdBackLabel->setText(color);

	color.sprintf("<table border = 1 bgcolor =\"#%02X%02X%02X",cbRedBox->value(),cbGreenBox->value(),cbBlueBox->value());
	color +=  "\"><tr><td>" + tr("BackColor of CF") + "</td></tr></table>";
	cfBackLabel->setText(color);
}

void FLUtil::setColor()
{
	nLink.setRgb(nlRedBox->value(),nlGreenBox->value(),nlBlueBox->value());
	cLink.setRgb(clRedBox->value(),clGreenBox->value(),clBlueBox->value());
	nBack.setRgb(nbRedBox->value(),nbGreenBox->value(),nbBlueBox->value());
	sBack.setRgb(sbRedBox->value(),sbGreenBox->value(),sbBlueBox->value());
	cBack.setRgb(cbRedBox->value(),cbGreenBox->value(),cbBlueBox->value());
	customBack = customBox->isChecked();
}

void FLUtil::reloadColor()
{
	nlRedBox->setValue(nLink.red());
	nlGreenBox->setValue(nLink.green());
	nlBlueBox->setValue(nLink.blue());
	clRedBox->setValue(cLink.red());
	clGreenBox->setValue(cLink.green());
	clBlueBox->setValue(cLink.blue());
	nbRedBox->setValue(nBack.red());
	nbGreenBox->setValue(nBack.green());
	nbBlueBox->setValue(nBack.blue());
	sbRedBox->setValue(sBack.red());
	sbGreenBox->setValue(sBack.green());
	sbBlueBox->setValue(sBack.blue());
	cbRedBox->setValue(cBack.red());
	cbGreenBox->setValue(cBack.green());
	cbBlueBox->setValue(cBack.blue());
}


const char FLUtil::indexToSize[] = { 10,12,16,18,20 };

