#include <qpe/config.h>
#include <qlayout.h>
#include <qcheckbox.h>

#include "FLib.h"
#include "CompareDialog.h"

CompareDialog::CompareDialog(QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
	if (FLib::isVga()) {
		resize( 200, 120 );
	} else {
		resize( 120,60);
	}
	setCaption( tr( "Compare Option" ) );
	
	layout = new QVBoxLayout(this);
	
	residentCheck = new QCheckBox(tr("Resident"),this);
	newCheck = new QCheckBox(tr("New"),this);
	oldCheck = new QCheckBox(tr("Old"),this);
	largeCheck = new QCheckBox(tr("Large"),this);
	smallCheck = new QCheckBox(tr("Small"),this);
	selOnlyCheck = new QCheckBox(tr("SelFileOnly"),this);
	dirCheck = new QCheckBox(tr("Select Directory"),this);
	layout->addWidget(residentCheck);
	layout->addWidget(newCheck);
	layout->addWidget(oldCheck);
	layout->addWidget(largeCheck);
	layout->addWidget(smallCheck);
	layout->addWidget(selOnlyCheck);
	layout->addWidget(dirCheck);

	Config cf = Config("FileLaunch");
	cf.setGroup("compare");
	setRes(cf.readBoolEntry("Resident",TRUE));
	setNew(cf.readBoolEntry("New",TRUE));
	setOld(cf.readBoolEntry("Old",FALSE));
	setLarge(cf.readBoolEntry("Large",FALSE));
	setSmall(cf.readBoolEntry("Small",FALSE));

	connect (residentCheck , SIGNAL(toggled(bool)) , this, SLOT(setRes(bool)) );
	connect (newCheck      , SIGNAL(toggled(bool)) , this, SLOT(setNew(bool)) );
	connect (oldCheck      , SIGNAL(toggled(bool)) , this, SLOT(setOld(bool)) );
	connect (largeCheck    , SIGNAL(toggled(bool)) , this, SLOT(setLarge(bool)) );
	connect (smallCheck    , SIGNAL(toggled(bool)) , this, SLOT(setSmall(bool)) );
}

CompareDialog::~CompareDialog()
{
}

void CompareDialog::accept()
{
	Config cf = Config("FileLaunch");
	cf.setGroup("compare");
	cf.writeEntry("Resident",residentCheck->isChecked());
	cf.writeEntry("New",newCheck->isChecked());
	cf.writeEntry("Old",oldCheck->isChecked());
	cf.writeEntry("Large",largeCheck->isChecked());
	cf.writeEntry("Small",smallCheck->isChecked());
	QDialog::accept();
}
void CompareDialog::setRes(bool b)
{
	residentCheck->setChecked(b);
}
void CompareDialog::setNew(bool b)
{
	if (b) {
		oldCheck->setChecked(FALSE);
	}
	newCheck->setChecked(b);
}
void CompareDialog::setOld(bool b)
{
	if (b) {
		newCheck->setChecked(FALSE);
	}
	oldCheck->setChecked(b);
}
void CompareDialog::setLarge(bool b)
{
	if (b) {
		smallCheck->setChecked(FALSE);
	}
	largeCheck->setChecked(b);
}
void CompareDialog::setSmall(bool b)
{
	if (b) {
		largeCheck->setChecked(FALSE);
	}
	smallCheck->setChecked(b);
}

int CompareDialog::cmpOpt()
{
	int r = 0;
	if (residentCheck->isChecked()) {
		r |= Resident;
	}
	if (newCheck->isChecked() ) {
		r |= FileNew;
	}
	if (oldCheck->isChecked() ) {
		r |= FileOld;
	}
	if (largeCheck->isChecked() ) {
		r |= FileLarge;
	}
	if (smallCheck->isChecked() ) {
		r |= FileSmall;
	}
	if (selOnlyCheck->isChecked()) {
		r |= SelFileOnly;
	}
	if (dirCheck->isChecked()) {
		r |= SelDirectory;
	}
	return r;
}
