#include <qapplication.h>
#include <qpe/mimetype.h>
#include <qpe/applnk.h>
#include <qfileinfo.h>
#include <qstringlist.h>

#include "FLib.h"

bool FLib::isVga()
{
	if (QApplication::desktop()->width() > HalfVgaWidth ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

bool FLib::isViewStyle()
{
	if (! isVga() ) {
		return TRUE;
	} else if (QApplication::desktop()->width() == VgaHeight ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

int FLib::fileType(const QFileInfo& f)
{
	if (f.isDir()) {
		return Directory;
	}
	MimeType mt(f.filePath());
	const AppLnk* app = mt.application();
	if (app != NULL) {
		QStringList mimes = app->mimeTypes();
		if (mimes.first() == QString("image/")+QString("*")) {
			return ImageFile;
		} else if (mimes.first() == "application/excel") {
			return ExcelFile;
		} else if (mimes.first() == "application/hancomword") {
			return WordFile;
		} else if (mimes.first() == "application/ipkg") {
			return IpkgFile;
		} else if (mimes.first() == "application/pdf") {
			return PdfFile;
		} else if (mimes.first() == "application/nf-url") {
			return HtmlFile;
		}
		const QString& ex =f.extension(FALSE).lower();
		if (ex == "lzh") {
			return LzhFile;
		} else if (ex == "zip") {
			return ZipFile;
		} else if (ex == "gz" || ex == "tgz") {
			return GZipFile;
		}
	}
	return TextFile;
}

QString FLib::fileTypeName(int i)
{
	return QString(ftname[i]);
}

const char* FLib::ftname[] = {
		"Directory",
		"text File",
		"html File",
		"ipkg File",
		"image File",
		"Word File",
		"Excel File"
		"lzh File",
		"zip File",
		"gzip File",
		"pdf File",
		"binary File"
};


QString FLib::formatNum(int i, int c)
{
	QString s = QString::number(i);
	for (int j = s.length() - 3 ; j > 0  ; j-= 3) {
		s.insert(j, ',' );
	}
	QString f ;
	f.fill(' ',c);
	s = f + s;
	return s.right(c);
}
QString FLib::fixNum(int i,int c)
{
	QString s = "000000" + QString::number(i);
	return s.right(c);
}

bool FLib::matchFileInfo(const QFileInfo& f1, const QFileInfo& f2)
{
	if (f1.filePath() != f2.filePath() ) {
		return false;
	} else if (f1.lastModified() != f2.lastModified() ) {
		return false;
	} else if (f1.size() != f2.size() ) {
		return false;
	}
	return true;
}
