#include <qheader.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qpe/qpeapplication.h>
#include <qpe/qcopenvelope_qws.h>
#include <qstringlist.h>
#include <qpe/mimetype.h>
#include <qpe/applnk.h>
#include <qpixmap.h>
#include <qiconset.h>
#include <qpe/resource.h>
#include <qstack.h>
#include <qmessagebox.h>
#include <qfont.h>
#include <qcombobox.h>
#include <qcstring.h>
#include <qtimer.h>

#include "FileLaunch.h"
#include "FileView.h"
#include "FLib.h"
#include "PreviewBox.h"
#include "ProcDialog.h"
#include "FilterDialog.h"
#include "CompareDialog.h"
#include "InputDialog.h"
#include "../shared/MiscLib.h"


FileLaunch::FileLaunch() : QMainWindow()
{
	setCaption(tr("FileLaunch")) ;
	setToolBarsMovable(false) ;


	base = new QWidget(this);

	loadCfg();


	stBar = statusBar();
	stLabel= new QLabel("",stBar);
	stLabel2 = new QLabel("",stBar);
	stLabel3 = new QLabel("",stBar);
	stBar->addWidget(stLabel);
	stBar->addWidget(stLabel2);
	stBar->addWidget(stLabel3);
	stBar->show();

	layout = new QHBoxLayout(base);
	if (direction) {
		layout->setDirection(QBoxLayout::LeftToRight);
	} else {
		layout->setDirection(QBoxLayout::Down);
	}

	file1 = new FileView(prevfile, "*",dispIcon ,hiddenMode,myZaurusMode,TRUE,base);
	file2 = new FileView(file2Path, "*",dispIcon ,hiddenMode,myZaurusMode,TRUE,base);


	connect(file1, SIGNAL(selectionChanged(const QFileInfo &)), this, SLOT(viewFile(const QFileInfo &)));
	connect(file2, SIGNAL(selectionChanged(const QFileInfo &)), this, SLOT(viewFile2(const QFileInfo &)));

	connect(file1, SIGNAL(dirChanged()), this, SLOT(showDir()));
	connect(file2, SIGNAL(dirChanged()), this, SLOT(showDir()));

	connect(file1, SIGNAL(fileSelected(const QFileInfo &)), this, SLOT(execFile(const QFileInfo  &)));
	connect(file2, SIGNAL(fileSelected(const QFileInfo &)), this, SLOT(execFile(const QFileInfo  &)));

	connect(file1, SIGNAL(hasFocused()) , this ,SLOT(f1Focused()) );
	connect(file2, SIGNAL(hasFocused()) , this ,SLOT(f2Focused()) );

	preview = new PreviewBox(base);
	setSize();
	preview->loadCfg();
	preview->setSmooth(smoothResize);

	makeMenu();

	layout->addWidget(file1) ;
	layout->addWidget(file2) ;
	layout->addWidget(preview) ;

	preview->hide();
	if (FLib::isVga()) {
		file1->setFont(QFont("lcfont",fileFontSize));
		file2->setFont(QFont("lcfont",fileFontSize));
		preview->setFontSize(viewFontSize);
	}

	f1focused = true;
	showDir();
	const QFileInfo* f = ((FileItem*)file1->currentItem())->info();
	viewFile(*f);
	setStBar();

	if (viewStBar) {
		stBar->show();
	} else {
		stBar->hide();
	}
	setScrollBar(viewScBar);

	setScrn();

	connect( qApp, SIGNAL(appMessage(const QCString&, const QByteArray&)),
		this, SLOT(appMessage(const QCString&, const QByteArray&)) );

	QCopChannel *channel = new QCopChannel("QPE/System", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(systemMessage(const QCString&, const QByteArray&)));

	setCentralWidget( base );
}

FileLaunch::~FileLaunch() 
{
}

void FileLaunch::closeEvent(QCloseEvent *e)
{
	prevfile = file1->currentDirName() +"/";
	file2Path = file2->currentDirName() + "/";
	saveCfg();
	e->accept();
}

void FileLaunch::setDocument(const QString &filename)
{
	qApp->processEvents();
	currentView()->setCurrentFile(filename);
	showDir();

}

void FileLaunch::appMessage(const QCString& msg, const QByteArray& data)
{
	QDataStream stream( data, IO_ReadOnly );
	if ( msg == "setFileSelection(QString,QString,QStringList)") {
		QString t,d;
		QStringList l;
		stream >> t >> d >> l;
		qApp->processEvents();
		currentView()->setFiles(t,d,l);
		showDir();
	}
}

void FileLaunch::resizeEvent(QResizeEvent *e)
{
	if (e){
		QMainWindow::resizeEvent(e);
	}
}

void FileLaunch::loadCfg()
{
	Config cf = Config("FileLaunch");
	cf.setGroup("config");
	prevfile = cf.readEntry("PrevFile",QPEApplication::documentDir());
	QFileInfo fi(prevfile);
	if ( (!fi.exists()) &&  (!QFile::exists(fi.dirPath())) ) {
		prevfile = QPEApplication::documentDir();
	}
	file2Path = cf.readEntry("File2Path",QPEApplication::documentDir());
	QFileInfo fi2(file2Path);
	if ( (!fi2.exists()) &&  (!QFile::exists(fi2.dirPath())) ) {
		file2Path = QPEApplication::documentDir();
	}

	hiddenMode = cf.readBoolEntry("HiddenMode",FALSE);
	myZaurusMode = cf.readBoolEntry("MyZaurus",FALSE);
	direction = cf.readBoolEntry("Direction",TRUE);
	disp2nd = cf.readBoolEntry("Disp2nd",TRUE);
	dispPreview = cf.readBoolEntry("PreviewMode",FALSE);
	addCost = cf.readNumEntry("AddCost",AddCost);
	viewStBar = cf.readBoolEntry("ViewStatusBar",TRUE);
	viewScBar = cf.readBoolEntry("ViewScrollBar",TRUE);
	shellExt = cf.readListEntry("ShellExtension",';');
	textExt = cf.readListEntry("TextExtension",';');
	memoAlter = cf.readEntry("MemoAlter");
	fileFontSize = cf.readNumEntry("FileFontSize",18);
	viewFontSize = cf.readNumEntry("ViewFontSize",18);
	smoothResize = cf.readBoolEntry("SmoothResize",true);
	symmetry = cf.readBoolEntry("SetSymmetry",false);
	splitPoint = cf.readNumEntry("SplitPoint",2);
	dispIcon = cf.readBoolEntry("DispIcon",true);
	maxRecent = cf.readNumEntry("MaxRecent",10);
	markAuto = cf.readNumEntry("MarkAuto",true);

	items.clear();
	QStringList li;
	cf.setGroup("menu");
	char keyMapping[MenuItem::Short_Size];
	memset(keyMapping,0,sizeof(keyMapping));
	QStringList errList;
	for (int i = 0 ; cf.hasKey("Item_" + FLib::fixNum(i,3)) ; ++ i) {
		li = cf.readListEntry("Item_" + FLib::fixNum(i,3),',');
		int hKey= li[10].toInt();
		if ( keyMapping[hKey] > 0 ) {
			li[10] = "0";
			hKey = 0;
			errList.append(li[3]);
			cf.writeEntry("Item_" + FLib::fixNum(i,3),li.join(","));
		} else if ( hKey != 0 ) {
			keyMapping[hKey] = 1;
		}
		items.append( new MenuItem(
					li[0].toInt(),li[1].toInt(),li[2].toInt(),
					li[3],li[4],li[5],
					li[6].toInt(),li[7].toInt(),
					li[8],
					li[9].toInt(),li[10].toInt()
					) );
	}
	if ( errList.count() > 0 ) {
		QMessageBox::warning(this,tr("config file error"),
			tr("Duplicate accel key :\n") + errList.join("\n")
			) ;
	}

	cf.setGroup("automark");
	QString markname = NULL;
	initCost = 0;
	for (int i = 0 ; i < MaxMark ; ++i) {
		markname  = cf.readEntry(QString("Mark%1").arg(i));
		if (! (markname.isEmpty() || markname.isNull()) ) {
			int c = cf.readNumEntry(QString("Cost%1").arg(i));
			markMap[markname] = c;
			if ( c < initCost || initCost == 0 ) {
				initCost = c;
			}
		}
	}
	
	cf.setGroup("recent");
	recent.clear();
	for (int i = 0 ; i < maxRecent ; ++i) {
		QString key = QString("Recent%1").arg(i);
		if ( cf.hasKey(key) ) {
			recent.append(cf.readEntry(key));
		}
	}
	
}

void FileLaunch::saveCfg()
{
	Config cf = Config("FileLaunch");
	cf.setGroup("config");
	cf.writeEntry("PrevFile",prevfile);
	cf.writeEntry("File2Path",file2Path);
	cf.writeEntry("HiddenMode",hiddenMode);
	cf.writeEntry("MyZaurus",myZaurusMode);
	cf.writeEntry("Direction",direction);
	cf.writeEntry("Disp2nd",disp2nd);
	cf.writeEntry("PreviewMode",dispPreview);
	cf.writeEntry("ViewStatusBar",viewStBar);
	cf.writeEntry("ViewScrollBar",viewScBar);
	cf.writeEntry("ShellExtension",shellExt.join(";"));
	cf.writeEntry("TextExtension",textExt.join(";"));
	cf.writeEntry("MemoAlter",memoAlter);
	cf.writeEntry("ViewFontSize",viewFontSize);
	cf.writeEntry("FileFontSize",fileFontSize);
	cf.writeEntry("SmoothResize",smoothResize);
	cf.writeEntry("SetSymmetry",symmetry);
	cf.writeEntry("SplitPoint",splitPoint);
//	cf.writeEntry("DispIcon",dispIcon);
	cf.writeEntry("MaxRecent",maxRecent);
//	cf.writeEntry("MarkAuto",markAuto);

	evalMark();
	cf.setGroup("automark");
	cf.clearGroup();
	int i = 0;
	for (MarkMap::Iterator it = markMap.begin() ; it != markMap.end() ; ++it) {
		markMap[it.key()] = markMap[it.key()] - 1;
		cf.writeEntry(QString("Mark%1").arg(i),it.key());
		cf.writeEntry(QString("Cost%1").arg(i),it.data());
		++i;
	}
	cf.setGroup("recent");
	cf.clearGroup();
	i = 0;
	for ( QStringList::Iterator it = recent.begin(); it != recent.end(); ++it ) {
		cf.writeEntry(QString("Recent%1").arg(i),(*it));
		++i;
	}
	
}

void FileLaunch::makeMenu()
{
	mb = new QPEMenuBar(this);
	fileMenu = new QPopupMenu(this);
	mb->insertItem( tr("&File"),fileMenu);

	viewMenu = new QPopupMenu(this);
	mb->insertItem( tr("&View"),viewMenu);

	execMenu = new QPopupMenu(this);
	mb->insertItem(tr("&Exec"),execMenu);

	optMenu = new QPopupMenu(this);
	mb->insertItem(tr("&Option"), optMenu);
	sizeMenu = new QPopupMenu(this);

	buildMenu();

	markMenu = new QPopupMenu(this);
	mb->insertItem(tr("&BookMark"),markMenu);

	connect(fileMenu , SIGNAL(aboutToShow()), this, SLOT( setFileMenu() ));
	connect(execMenu , SIGNAL(aboutToShow()), this, SLOT( setExecMenu() ));
	connect(markMenu , SIGNAL(aboutToShow()), this, SLOT( setMarkMenu() ));

}

void FileLaunch::buildMenu()
{
	fileMenu->clear();
	setMenu(fileMenu,CopyMenu);
	setMenu(fileMenu,MoveMenu);
	setMenu(fileMenu,DelMenu);
	setMenu(fileMenu,RenameMenu);
	setMenu(fileMenu,MDMenu);
	setMenu(fileMenu,TarMenu);
	setMenu(fileMenu,ExtMenu);
	setMenu(fileMenu,LnkMenu);
	setMenu(fileMenu,ModMenu);
	setMenu(fileMenu,OwnMenu);

	viewMenu->clear();
	setMenuItem(viewMenu,SLOT(reload()) ,ReloadMenu);
	setMenuItem(viewMenu,SLOT(setFilter()) , FilterMenu);
	setMenuItem(viewMenu,SLOT(cmpFile()),CompareMenu);
	setMenuItem(viewMenu,SLOT(selAll()),SelAllMenu);
	setMenuItem(viewMenu,SLOT(swapView()),SwapViewMenu);
	setMenuItem(viewMenu,SLOT(callProperty()),PropertyMenu);
	setMenuItem(viewMenu,SLOT(selectInvert()),SelInvMenu);
	setMenuItem(viewMenu,SLOT(disp2ndView()),SecondViewMenu);
	setMenuItem(viewMenu,SLOT(matchDirectory()),MatchDirMenu);
	setMenuItem(viewMenu,SLOT(viewStatusBar()),StatusBarMenu);
	setMenuItem(viewMenu,SLOT(viewScrollBar()),ScrollBarMenu);
	setMenuItem(viewMenu,SLOT(viewRecent()),RecentMenu);

	viewMenu->setItemChecked(viewMenu->idAt(Disp2nd),disp2nd);
	viewMenu->setItemChecked(viewMenu->idAt(StatusBar),viewStBar);
	viewMenu->setItemChecked(viewMenu->idAt(ScrollBar),viewScBar);

	optMenu->clear();
	setMenuItem(optMenu,SLOT(setPreview()),PreviewMenu);
	setMenuItem(optMenu,SLOT(setResize()),ResizeMenu);
//	optMenu->insertItem( menuName(items.at(DirectionMenu)), sizeMenu);
//	setMenuItem(optMenu,SLOT(setDirection()),DirectionMenu);
	setMenuMenu(optMenu,sizeMenu,DirectionMenu);
	setMenuItem(optMenu,SLOT(setMyZaurus()),MyZaurusMenu);
	setMenuItem(optMenu,SLOT(setHidden()),HiddenFileMenu);
	setMenuItem(optMenu,SLOT(addMarkManual()),AddMarkMenu);
	setMenuItem(optMenu,SLOT(callMark()),CallMarkMenu);
	setMenuItem(optMenu,SLOT(callOption()),OptionMenu);
	
	optMenu->setItemChecked(optMenu->idAt(Preview),dispPreview);
	optMenu->setItemChecked(optMenu->idAt(Resize),smoothResize);
	optMenu->setItemChecked(optMenu->idAt(MyZaurus),myZaurusMode);
	optMenu->setItemChecked(optMenu->idAt(Hidden),hiddenMode);

	sizeMenu->clear();
	setMenuItem(sizeMenu,SLOT(setDirection()),HorizontalMenu);
//	sizeMenu->insertItem(tr("Horizontal"),this,SLOT(setDirection()));
	for (int i = 0; i < 5 ; ++ i) {
//		setMenuItemId(sizeMenu,SLOT(setSplit(int)),Minus2Menu+i,i);
		sizeMenu->insertItem( QString("%1").arg(i-2,2),this,SLOT( setSplit(int)),0,i);
	}
	setMenuItem(sizeMenu,SLOT(setViewPlus()),ViewPlusMenu);
	setMenuItem(sizeMenu,SLOT(setViewMinus()),ViewMinusMenu);

	sizeMenu->setItemChecked(sizeMenu->idAt(0),direction);
	sizeMenu->setItemChecked(sizeMenu->idAt(splitPoint+SplitBase),TRUE);

	execMenu->clear();
	execMenu->insertItem(tr("&Exec") , this ,SLOT(execFile()) );
	execMenu->insertSeparator(1);

	QListIterator<MenuItem> it(items);
	QPopupMenu* menuData = execMenu;
	int lastLev = 0;
	QStack<QPopupMenu> mstack;
	eMenus.clear();
	eMenus.append(execMenu);
	for (int i = 0 ; i < MenuItem::reserveItem ; ++it, ++i) {
		;
	}
	
	for (int i = 0 ; it.current(); ++it ,++i) {
		MenuItem* m = it.current();
		if (m->depth() < lastLev) {
			for (int j = lastLev ; j > m->depth() ; --j) {
				if (! mstack.isEmpty()) {
					menuData = mstack.pop();
				}
			}
		}
		lastLev = m->depth();
		switch (m->type()) {
		case MenuItem::Menu:
		{
			QPopupMenu *p = new QPopupMenu(menuData);
			menuData->insertItem(menuName(m),p,i+MenuItem::reserveItem);
			eMenus.append(p);
			mstack.push(menuData);
			menuData = p;
			break;
		}
		case MenuItem::QtApps:
		{
			const AppLnk *app= new AppLnk(m->pathName());
			menuData->insertItem(QIconSet(app->pixmap()),menuName(m), this, SLOT(menuExec(int)),MenuItem::hkeyCode(m->hotKey()),i+MenuItem::reserveItem);
			delete app;
			break;
		}
		case MenuItem::QCopSend:
		{
			menuData->insertItem(QIconSet(Resource::loadPixmap( "alarm_s" )),menuName(m), this, SLOT(menuExec(int)),MenuItem::hkeyCode(m->hotKey()),i+MenuItem::reserveItem);
			break;
		}
		default:
		{
			menuData->insertItem(QIconSet(Resource::loadPixmap( "konsole" )),menuName(m), this, SLOT(menuExec(int)),MenuItem::hkeyCode(m->hotKey()),i+MenuItem::reserveItem);
			break;
		}
		}
	}
}


FileView* FileLaunch::currentView()
{
	if (! f1focused) {
		return file2;
	} else {
		return file1;
	}
}

void FileLaunch::callProperty()
{
	const QFileInfo* file = ((FileItem*)currentView()->currentItem())->info();
	QString pr = MiscLib::makeProperty(*file,"");
	QMessageBox::information(this, tr("Property"),pr) ;
}

void FileLaunch::menuExec(int i)
{
	QStringList p1;
	QStringList p2;
	switch (items.at(i)->type()) {
	case MenuItem::QtApps:
		{
			QString arg ;
			if (items.at(i)->p1Type() == MenuItem::Inputd) {
				if (setArg(i ,p1 ,p2)) {
					arg = p2[0];
				} else {
					break;
				}
			} else {
				FileItem* ff = (FileItem*)currentView()->currentItem();
				setUseFile(*(ff->info()));
				arg = ff->info()->filePath();
			}
			addRecent(arg);
			const AppLnk *app= new AppLnk(items.at(i)->pathName());
			app->execute(arg);
			delete app;
		}
		break;
	case MenuItem::QCopSend:
		{
			QString arg;
			if (items.at(i)->p1Type() == MenuItem::Inputd) {
				if (setArg(i ,p1 ,p2)) {
					arg = p2[0];
				} else {
					break;
				}
			} else {
				FileItem* ff = (FileItem*)currentView()->currentItem();
				setUseFile(*(ff->info()));
				arg = ff->info()->filePath();
			}
			QCString ch = "QPE/"+ QCString(items.at(i)->pathName());
			QCString msg = QCString(items.at(i)->paramForm());
			{
				QCopEnvelope e(ch, msg);
				e << arg;
			}
		}
		break;
	case MenuItem::Menu:
		break;
	default:
		{
			FileItem* f1 = (FileItem*)(primaryView()->currentItem());
			FileItem* f2 = (FileItem*)(secondaryView()->currentItem());

			if (setArg(i ,p1 ,p2)) {
				QString para =items.at(i)->paramForm();
				para.replace(QRegExp("\\$1"),"\"" + p1.join("\" \"") + "\"");
				para.replace(QRegExp("\\$2"),"\"" + p2.join("\" \"") + "\"");
	
				ProcDialog p(items.at(i)->pathName() ,para,viewFontSize,this);
				p.exec();
				// reload 
				reloadView(f1);
				reloadView(f2);
			}
		}
		break;
	}
}

void FileLaunch::reloadView(FileItem* fi)
{
	FileView* file = (FileView*) (fi->listView());
	file->reloadView(fi);
}

bool FileLaunch::setArg(int i , QStringList& p1,QStringList& p2)
{
	FileView* fv;
	if (items.at(i)->p2Type() == MenuItem::NotUseParam) {
		fv = currentView();
	} else {
		fv = primaryView();
	}
	int p1type = items.at(i)->p1Type();
	bool special = false;
	if (p1type == MenuItem::Inputd && items.at(i)->p2Type() == MenuItem::NotUseParam ) {
		special = true;
		p1type = MenuItem::Always;
	}
	p1 = expandFile(p1type,fv,items.at(i)->msgDisp());
	if (p1.count() == 0) {
		return false;
	}
	if (special ) {
		p2 = expandFile(items.at(i)->p1Type(),fv,items.at(i)->msgDisp());
		if (p2.count() == 0 ) {
			return false;
		}
	} else if (items.at(i)->p2Type() != MenuItem::NotUseParam && disp2nd) {
		p2 = expandFile(items.at(i)->p2Type(),secondaryView(),items.at(i)->msgDisp());
		if (p2.count() == 0 ) {
			return false;
		}
	}
	if (items.at(i)->type() == MenuItem::ShellW ) {
		QString wm = tr("exec command :") + items.at(i)->name() + tr(" \n") ;
		if (p1.count() > ListMax) {
			wm += p1.first() + "\n" + "....\n" + p1.last() +"\n";
		} else {
			wm += p1.join("\n") + "\n";
		}
		if (p2.count() > 0) {
			wm += tr("to\n");
			if (p2.count() > 3) {
				wm += p2.first() + "\n" + "....\n" + p2.last() +"\n";
			} else {
				wm += p2.join("\n");
			}
			wm += tr("\n");
		}
		if ( fv->selHasDir() ||
			(items.at(i)->p2Type() != MenuItem::NotUseParam && secondaryView()->selHasDir() ) ){
//		if ( file1->selHasDir() || file2->selHasDir() ) {
			wm += tr("\nThere are some Directory in select\n");
		}
		if (QMessageBox::warning(this,items.at(i)->name()
			,wm,tr("Yes"),tr("No"),QString::null,0,1)  == 1 ) {
			return false;
		}
	}
	if (items.at(i)->p2Type() != MenuItem::NotUseParam) {
		setUseFile(QFileInfo(p2[0]));
	}
	if (p1type != MenuItem::Separate) {
		setUseFile(QFileInfo(p1[0]));
	} else {
		setUseFile(QFileInfo(p1[0]+ "/" + p1[1]));
	}
	return true;
}

QStringList FileLaunch::expandFile(int type,FileView* file,const QString& cap )
{
	QStringList para;
	QString dir = "";

	QValueList<QFileInfo>& fi= file->multiSelected();
	if (type == MenuItem::Separate) {
		if (file->listType() == FileView::Selection ) {
			if (fi.count() == 0 || fi[0].filePath() == file->baseDir()) {
				return para;
			}
			para.append(file->baseDir());
			for ( uint i = 0 ; i < fi.count() ; ++ i) {
				para.append(file->dropBase(fi[i]));
			}
			return para;
		}
		if (fi.count() == 0 || fi[0].filePath().right(3) == "/..") {
			return para;
		}
		para.append(fi[0].dirPath());
		for ( uint i = 0 ; i < fi.count() ; ++ i) {
			para.append(fi[i].fileName());
		}
		return para;
	}
	if (fi.count() == 0) {
		return para;
	} else {
		for ( uint i = 0 ; i < fi.count() ; ++ i) {
			QString fr = fi[i].filePath();
			if ( fr.right(3) == "/.." ) {
				if (fi.count() == 1 ) {
					para.append(fr.left(fr.length()-3));
				}
			} else {
				para.append(fr);
			}
		}
		dir = fi[0].dirPath() + "/";
	}
	if (type == MenuItem::Inputs || type == MenuItem::Inputd) {
		QString msg;
		if (fi.count() > 0  ) {
			msg = fi[0].fileName();
		} else {
			msg = "";
		}
		if (msg == "..") {
			msg = "";
		}
		InputDialog d(cap,msg,this);
		int r = d.exec();
		if (r == QDialog::Rejected) {
			para.clear();
			return para;
		}
		para.clear();
		if (type == MenuItem::Inputs ) {
			para.append(dir + d.text());
		} else {
			para.append(d.text());
		}
	}
	return para;
}

void FileLaunch::setFileMenu()
{
	setMenuEnable(fileMenu);
}

void FileLaunch::setExecMenu()
{
	const QFileInfo fi = *(currentView()->selected());
	MimeType mt(fi.filePath());
	const AppLnk* app = mt.application();
	if (app == NULL) {
		QString ext = fi.extension();
		if (ext.length() == 0 ) {
			ext = "()";
		}
		if (shellExt.findIndex(ext) > -1 && fi.isFile() && fi.isExecutable()) {
			execMenu->changeItem(execMenu->idAt(0),QIconSet(Resource::loadPixmap( "konsole" )),fi.fileName());
		} else if (textExt.findIndex(ext) > -1 && memoAlter.length() > 0) {
			const AppLnk *app= new AppLnk(memoAlter);
			execMenu->changeItem(execMenu->idAt(0),QIconSet(app->pixmap()),app->name() );
			delete app;

		} else {
			execMenu->changeItem(execMenu->idAt(0),QIconSet(),tr("Not Executable"));
		}
	} else {
		if (app->exec() == "textedit" && memoAlter.length() > 0) {
			const AppLnk *app= new AppLnk(memoAlter);
			execMenu->changeItem(execMenu->idAt(0),QIconSet(app->pixmap()),app->name() );
			delete app;
		} else {
			execMenu->changeItem(execMenu->idAt(0),QIconSet(app->pixmap()),app->name() );
		}
	}
	for (QListIterator<QPopupMenu> it(eMenus) ; it.current() ; ++it) {
		setMenuEnable(it.current());
	}
}

void FileLaunch::setMenuEnable(QPopupMenu* current)
{
	for (uint i = 0 ; i < current->count() ; ++i ) {
		int id = current->idAt(i);
		if (id >= 0 ) {
			MenuItem* m = items.at(id);
			if (m->type() != MenuItem::Menu) {
				int p1 = m->p1Type();
				int p2 = m->p2Type();
				if (p2 == MenuItem::NotUseParam ) {
					int pc  = ! f1focused ? p2 : p1;
					if ( pc == MenuItem::DirOnly && ! currentView()->selIsDir() ) {
						current->setItemEnabled(id,FALSE);
					} else if ( pc == MenuItem::FileOnly && ! currentView()->selIsFile() ) {
						current->setItemEnabled(id,FALSE);
					} else {
						current->setItemEnabled(id,TRUE);
					}
				} else {
					if (secondaryView()->listType() == FileView::Selection) {
						current->setItemEnabled(id,FALSE);
					} else if (secondaryView()->selFileCount() > 1) {
						current->setItemEnabled(id,FALSE);
					} else if (p1 == MenuItem::DirOnly && ! primaryView()->selIsDir() ) {
						current->setItemEnabled(id,FALSE);
					} else if (p1 == MenuItem::FileOnly && ! primaryView()->selIsFile() ) {
						current->setItemEnabled(id,FALSE);
					} else if (p2 == MenuItem::DirOnly && ! secondaryView()->selIsDir() ) {
						current->setItemEnabled(id,FALSE);
					} else if (p2 == MenuItem::FileOnly && ! secondaryView()->selIsFile() ) {
						current->setItemEnabled(id,FALSE);
					} else if (!disp2nd) {
						current->setItemEnabled(id,FALSE);
					} else {
						current->setItemEnabled(id,TRUE);
					}
				}
			}
		}
	}
}

void FileLaunch::viewFile(const QFileInfo & fi)
{
//	if (dispPreview) {
//		setUseFile(fi.filePath());
		preview->setFile(fi);
//	}
	setStBar();
	setScrn();
}
void FileLaunch::viewFile2(const QFileInfo & fi)
{
	setStBar();
	setScrn();
}
void FileLaunch::execFile(const QFileInfo & fi)
{
	setUseFile(fi.filePath());
	MimeType mt(fi.filePath());
	const AppLnk* app = mt.application();
	if (app != NULL) {
		if (app->exec() == "textedit" && memoAlter.length() > 0) {
			const AppLnk *app= new AppLnk(memoAlter);
			app->execute(fi.filePath());
			delete app;
		} else {
			DocLnk doc( fi.filePath(), FALSE );
			doc.execute();
		}
		addRecent(fi.filePath());
	} else {
		QString ext = fi.extension();
		if (ext.length() == 0 ) {
			ext = "()";
		}
		if (textExt.findIndex(ext) > -1 && memoAlter.length() > 0) {
			const AppLnk *app= new AppLnk(memoAlter);
			app->execute(fi.filePath());
			delete app;
			addRecent(fi.filePath());
		} else if (shellExt.findIndex(ext) > -1 && fi.isFile() && fi.isExecutable()) {
			QString para = "";
			FileItem* f1 = (FileItem*)file1->currentItem();
			FileItem* f2 = (FileItem*)file2->currentItem();
			FileView* fv;
			FileItem* ft;
			if (f1focused) {
				fv = file2;
				ft = f2;
			} else {
				fv = file1;
				ft = f1;
			}
			if (fv->selFileCount() == 1) {
				para = ft->info()->filePath();
			}
			if (QMessageBox::warning(this,tr("Execute File"),
				fi.fileName() + "\n" + para + "\n",
				tr("Yes"),tr("No"),QString::null,0,1)  == 1 ) {
				return;
			}
			ProcDialog p(fi.filePath() ,para,viewFontSize,this);
			p.exec();
			// reload 
			reloadView(f1);
			reloadView(f2);
		}
	}
}
void FileLaunch::execFile()
{
	execFile(*(currentView()->selected()));
}
void FileLaunch::showDir()
{
	setCaption(currentView()->currentViewPath() + tr(" - FileLaunch"));
	setStBar();
}
void FileLaunch::reload()
{
	reloadView((FileItem*)currentView()->currentItem());
//	currentView()->updateView();
}
void FileLaunch::setFilter()
{
	FilterDialog d(this);
	int r = d.exec();
	if (r) {
		QString fil = d.filterBox->currentText();
		currentView()->setNameFilter(fil);
		currentView()->updateView();
		setStBar();
	}

}
void FileLaunch::cmpFile()
{
	CompareDialog d(this);
	int r = d.exec();
	if (r ) {
		int op = d.cmpOpt();
		if (op > 0 ) {
			QValueList<QFileInfo> f1All;
			if (op & CompareDialog::SelFileOnly) {
				f1All = primaryView()->multiSelected();
			} else {
				f1All = primaryView()->allFiles(!(op & CompareDialog::SelDirectory));
			}
			QValueList<QFileInfo> f2All = secondaryView()->allFiles(!(op & CompareDialog::SelDirectory));
			primaryView()->clearSelection();
			QValueList<QFileInfo> selFiles = doCompare(op,f1All,f2All);
			primaryView()->fileSpecify(selFiles);
		}
	}
}

QValueList<QFileInfo> FileLaunch::doCompare(int op, QValueList<QFileInfo>& f1,QValueList<QFileInfo>& f2) const
{
	QValueList<QFileInfo> sel;
	QValueList<QFileInfo>::Iterator it1;
	QValueList<QFileInfo>::Iterator it2;

	for (it1 = f1.begin() ; it1 != f1.end() ; ++it1) {
		it2 = f2.begin();
		for ( ; it2 != f2.end() ; ++it2) {
			if ( (*it1).fileName() == (*it2).fileName() ) {
				bool aped = false;
				if (op & CompareDialog::FileNew) {
					if ( (*it1).lastModified() > (*it2).lastModified() ) {
						aped = true;
						sel.append(*it1);
					}
				} else if (op & CompareDialog::FileOld) {
					if ( (*it1).lastModified() < (*it2).lastModified() ) {
						aped = true;
						sel.append(*it1);
					}
				}
				if ( ! aped ) {
					if (op & CompareDialog::FileLarge) {
						if ( (*it1).size() > (*it2).size() ) {
							sel.append(*it1);
						}
					} else if (op & CompareDialog::FileSmall) {
						if ( (*it1).size() < (*it2).size() ) {
							sel.append(*it1);
						}
					}
				}
				break;
			}
		}
		if (it2 == f2.end() ) {
			if (op & CompareDialog::Resident) {
				sel.append(*it1);
			}
		}
	}

	return sel;
}

void FileLaunch::f1Focused()
{
	file1->triggerUpdate();
	f1focused = true;
	showDir();
	file2->triggerUpdate();
}
void FileLaunch::f2Focused()
{
	file2->triggerUpdate();
	f1focused = false;
	showDir();
	file1->triggerUpdate();
}
void FileLaunch::callOption()
{
	saveCfg();
	QCopEnvelope e("QPE/System", "execute(QString)");
		e << QString("flutil");
}

void FileLaunch::setScrn()
{
	if (dispPreview) {
		file2->hide();
		preview->show();
	} else {
		preview->hide();
		if (disp2nd) {
			file2->show();
		} else {
			file2->hide();
		}
	}
}
void FileLaunch::disp2ndView()
{
	if (disp2nd) {
		viewMenu->setItemChecked(viewMenu->idAt(Disp2nd),FALSE);
		disp2nd = FALSE;
	} else {
		viewMenu->setItemChecked(viewMenu->idAt(Disp2nd),TRUE);
		disp2nd = TRUE;
	}
	setScrn();
}

void FileLaunch::setMyZaurus()
{
	if (myZaurusMode) {
		optMenu->setItemChecked(optMenu->idAt(MyZaurus),FALSE);
		myZaurusMode = FALSE;
	} else {
		optMenu->setItemChecked(optMenu->idAt(MyZaurus),TRUE);
		myZaurusMode = TRUE;
	}
	file1->setMyZaurus(myZaurusMode);
	file2->setMyZaurus(myZaurusMode);
	showDir();

}
void FileLaunch::setHidden()
{
	if (hiddenMode) {
		optMenu->setItemChecked(optMenu->idAt(Hidden),FALSE);
		hiddenMode = FALSE;
	} else {
		optMenu->setItemChecked(optMenu->idAt(Hidden),TRUE);
		hiddenMode = TRUE;
	}
	file1->setHidden(hiddenMode);
	file2->setHidden(hiddenMode);
}
void FileLaunch::setPreview()
{
	if (dispPreview) {
		optMenu->setItemChecked(optMenu->idAt(Preview),FALSE);
		dispPreview = FALSE;
	} else {
		optMenu->setItemChecked(optMenu->idAt(Preview),TRUE);
		dispPreview = TRUE;
	}
//	viewFile(*(((FileItem*)file1->currentItem())->info()));
	setScrn();
}
void FileLaunch::setResize()
{
	if (smoothResize) {
		optMenu->setItemChecked(optMenu->idAt(Resize),FALSE);
		smoothResize = FALSE;
	} else {
		optMenu->setItemChecked(optMenu->idAt(Resize),TRUE);
		smoothResize = TRUE;
	}
	preview->setSmooth(smoothResize);
}
void FileLaunch::selAll()
{
	currentView()->selAll();
}

void FileLaunch::swapView()
{
	QString f1path = file1->selected()->filePath();
	QString f2path = file2->selected()->filePath();
	file1->setCurrentFile(f2path);
	file2->setCurrentFile(f1path);
	showDir();
}

void FileLaunch::setUseFile(const QFileInfo& f)
{
	if ( ! markAuto ) {
		return;
	}
	QString markPath = f.dirPath() + "/";
	if (markPath == "./") {
		markPath = "/";
	}
	if (markMap.contains(markPath)) {
		markMap[markPath] = markMap[markPath] + addCost;
	} else {
		markMap.insert(markPath,initCost);
	}
}

void FileLaunch::gotoMark(int i)
{
	QString path = markMenu->text(i);
	currentView()->setCurrentFile(path);
	showDir();
}

void FileLaunch::setMarkMenu()
{
	QStringList mark = evalMark();
	markMenu->clear();
	for (QStringList::Iterator it= mark.begin() ; it != mark.end() ; ++it) {
		markMenu->insertItem(*it , this , SLOT(gotoMark(int)) );
	}
	saveCfg();
}

QStringList FileLaunch::evalMark()
{
	QStringList list;
	int costMax = 0;
	int costMin = 0;
	for (MarkMap::Iterator it = markMap.begin() ; it != markMap.end() ; ++it) {
		if (it.data() > costMax) {
			costMax = it.data();
		}
		if (it.data() < costMin || costMin == 0) {
			costMin = it.data();
		}
		if (list.count() > 0) {
			QStringList::Iterator si = list.begin();
			for ( ; si != list.end() ; ++si) {
				if (it.data() > markMap[*si] ) {
					list.insert(si,it.key());
					break;
				}
			}
			if (si == list.end() ) {
				list.append(it.key());
			}
		} else {
			list.append(it.key());
		}
	}
	if ((int)list.count() > MaxMark) {
		QStringList::Iterator it = list.begin();
		for (int i = 0; i < MaxMark ; ++i ) {
			++it;
		}
		while (it != list.end() ) {
			 markMap.remove(*it);
			 it = list.remove(it) ;
		}
	}
	if (costMax > MaxCost) {
		for (MarkMap::Iterator it = markMap.begin() ; it != markMap.end() ; ++it) {
			markMap[it.key()] = it.data() / 10;
		}
	}
	initCost = costMin + InitCost;
	return list;
}

void FileLaunch::systemMessage(const QCString& msg, const QByteArray& data)
{
	QDataStream stream(data, IO_ReadOnly);
	QString ap = "";
	if (msg == "closing(QString)" ) {
		stream >> ap;
		if (ap == "flutil") {
			reloadCfg();
		}
	}
}

void FileLaunch::reloadCfg()
{
	loadCfg();
	if (FLib::isVga()) {
		file1->setFont(QFont("lcfont",fileFontSize));
		file2->setFont(QFont("lcfont",fileFontSize));
		preview->setFontSize(viewFontSize);
	}
	preview->loadCfg();
	file1->loadCfg();
	file2->loadCfg();

	buildMenu();
}

void FileLaunch::selectInvert()
{
	currentView()->selectInvert();
}

void FileLaunch::matchDirectory()
{
	QString path = currentView()->selected()->filePath();
	if (f1focused) {
		file2->setCurrentFile(path);
	} else {
		file1->setCurrentFile(path);
	}
}

void FileLaunch::setStBar()
{
	stLabel->setText(currentView()->selectedMsg());
	stLabel2->setText(tr(" Filter ")+currentView()->currentFilter() + " ");
	if (dispPreview) {
		stLabel3->setText(" " + preview->prevFileCode());
	} else {
		stLabel3->setText("");
	}
}

void FileLaunch::viewStatusBar()
{
	if (viewStBar) {
		viewMenu->setItemChecked(viewMenu->idAt(StatusBar),FALSE);
		viewStBar = FALSE;
		stBar->hide();
	} else {
		viewMenu->setItemChecked(viewMenu->idAt(StatusBar),TRUE);
		viewStBar = TRUE;
		stBar->show();
	}
}
void FileLaunch::viewScrollBar()
{
	if (viewScBar) {
		viewMenu->setItemChecked(viewMenu->idAt(ScrollBar),FALSE);
		viewScBar = FALSE;
		setScrollBar(FALSE);
	} else {
		viewMenu->setItemChecked(viewMenu->idAt(ScrollBar),TRUE);
		viewScBar = TRUE;
		setScrollBar(TRUE);
	}
}
void FileLaunch::setScrollBar(bool b)
{
	if (b) { 
		file1->setVScrollBarMode(QScrollView::Auto);
		file2->setVScrollBarMode(QScrollView::Auto);
		file1->setHScrollBarMode(QScrollView::Auto);
		file2->setHScrollBarMode(QScrollView::Auto);
	} else {
		file1->setVScrollBarMode(QScrollView::AlwaysOff);
		file2->setVScrollBarMode(QScrollView::AlwaysOff);
		file1->setHScrollBarMode(QScrollView::AlwaysOff);
		file2->setHScrollBarMode(QScrollView::AlwaysOff);
	}
}
QString FileLaunch::menuName(MenuItem* m)
{
	QString s = m->name();
	if (m->shortKey() > 0) {
		s += "(&" + MenuItem::skeyCode(m->shortKey()) + ")";
	}
	return s;
}

void FileLaunch::callMark()
{
	markMenu->exec();
}

void FileLaunch::addMarkManual()
{
	setUseFile(*((FileItem*)(currentView()->currentItem()))->info());

}

FileView* FileLaunch::primaryView()
{
	if (symmetry && (! f1focused)) {
		return file2;
	} else {
		return file1;
	}
}

FileView* FileLaunch::secondaryView()
{
	if (symmetry && (! f1focused)) {
		return file1;
	} else {
		return file2;
	}
}

void FileLaunch::setDirection()
{
	if (direction) {
		sizeMenu->setItemChecked(sizeMenu->idAt(0),FALSE);
		direction = FALSE;
		setSize();
		layout->setDirection(QBoxLayout::Down);
	} else {
		sizeMenu->setItemChecked(sizeMenu->idAt(0),TRUE);
		direction = TRUE;
		setSize();
		layout->setDirection(QBoxLayout::LeftToRight);
	}
}
void FileLaunch::setSplit(int i)
{
	sizeMenu->setItemChecked(sizeMenu->idAt(splitPoint+SplitBase),FALSE);
	sizeMenu->setItemChecked(sizeMenu->idAt(i+SplitBase),TRUE);
	splitPoint = i;
	setSize();
	layout->activate();
}

void FileLaunch::setSize()
{
	if (direction) {
		file1->setSize(file1Width(splitPoint),100);
		file2->setSize(file2Width(splitPoint),100);
		preview->setSize(file2Width(splitPoint),100);
	} else {
		file1->setSize(100,file1Height(splitPoint));
		file2->setSize(100,file2Height(splitPoint));
		preview->setSize(100,file2Height(splitPoint));
	}
}

void FileLaunch::setViewPlus()
{
	if (splitPoint < SplitMax ) {
		setSplit(splitPoint + 1 );
	}
}

void FileLaunch::setViewMinus()
{
	if (splitPoint > 0 ) {
		setSplit(splitPoint - 1 );
	}
}

void FileLaunch::addRecent(QString s)
{
	if (recent.find(s) == recent.end() ) {
		recent.append(s);
		if ( recent.count() > maxRecent ) {
			recent.remove(recent.begin());
		}
	}
}

void FileLaunch::viewRecent()
{
	currentView()->setFiles(tr("Recent Files") ,currentView()->currentPathName(),recent);
	showDir();
}
