#include <qmainwindow.h>
#include <qpe/qpemenubar.h>
#include <qapplication.h>
#include <qpopmenu.h>
#include <qlayout.h>
#include <qstring.h>
#include <qlist.h>
#include <qfileinfo.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qmap.h>

#include "MenuItem.h"

class FileView;
class FileItem;
class PreviewBox;
typedef QMap<QString,int> MarkMap;

	struct compInfo {
		QString fileName;
		int f1Size;
		QDateTime f1Date;
		int f2Size;
		QDateTime f2Date;
	};

class FileLaunch : public QMainWindow
{
	Q_OBJECT

public:
	FileLaunch() ;
	~FileLaunch() ;

	enum ReservedArea {
		CopyMenu,
		MoveMenu,
		DelMenu,
		RenameMenu,
		MDMenu,
		TarMenu,
		ExtMenu,
		LnkMenu,
		ModMenu,
		OwnMenu,
		FindMenu,
		ReserveMenu1,
		ReserveMenu2,
		ReloadMenu,
		FilterMenu,
		CompareMenu,
		SelAllMenu,
		SelInvMenu,
		SwapViewMenu,
		MatchDirMenu,
		PropertyMenu,
		StatusBarMenu,
		ScrollBarMenu,
		SecondViewMenu,
		PreviewMenu,
		HiddenFileMenu,
		DirectionMenu,
		MyZaurusMenu,
		AddMarkMenu,
		CallMarkMenu,
		OptionMenu,
		ResizeMenu,
		HorizontalMenu,
		ViewPlusMenu,
		ViewMinusMenu,
		RecentMenu,
		ReserveMax
	};

protected:
	virtual void resizeEvent(QResizeEvent *e);
	virtual void closeEvent(QCloseEvent *e);

private slots:
	void setDocument(const QString &s) ;
	void appMessage ( const QCString & msg, const QByteArray & data );
	void callProperty();

	void setMyZaurus();
	void setHidden();
	void setDirection();
	void setPreview();
	void setResize();

	void menuExec(int i);
	void setSplit(int i);
	void setViewPlus();
	void setViewMinus();
	void setExecMenu();
	void setFileMenu();
	void viewFile(const QFileInfo& fi);
	void viewFile2(const QFileInfo& fi);
	void execFile(const QFileInfo& fi);
	void execFile();
	void showDir();
	void reload();
	void setFilter();
	void cmpFile();
	void selAll();
	void swapView();
	void f1Focused();
	void f2Focused();
	void callOption();
	void disp2ndView();
	void gotoMark(int i);
	void setMarkMenu();
	void systemMessage(const QCString& msg, const QByteArray& data);
	void selectInvert();
	void matchDirectory();
	void viewStatusBar();
	void viewScrollBar();
	void callMark();
	void addMarkManual();
	void viewRecent();

private:
	QWidget *base;
	FileView *file1;
	FileView *file2;
	PreviewBox *preview;
	QBoxLayout* layout;

	QList<MenuItem> items;
	QList<QPopupMenu> eMenus;

	QPEMenuBar *mb;
	QPopupMenu *fileMenu;
	QPopupMenu *viewMenu;
	QPopupMenu *execMenu;
	QPopupMenu *optMenu;
	QPopupMenu *sizeMenu;
	QPopupMenu *markMenu;
	
	QStatusBar *stBar;
	QLabel* stLabel;
	QLabel* stLabel2;
	QLabel* stLabel3;

	QString prevfile;
	QString file2Path;
	bool hiddenMode;
	bool myZaurusMode;
	bool direction;
	bool dispPreview;
	bool disp2nd;
	bool previewMode;
	bool viewStBar;
	bool viewScBar;
	int viewFontSize;
	int fileFontSize;
	bool smoothResize;
	bool symmetry;
	int splitPoint;
	bool dispIcon;
	bool markAuto;

	MarkMap markMap;
	static const int MaxCost = 30000;
	static const int MaxMark = 15;
	static const int InitCost = 100;
	static const int AddCost = 50;
	static const uint ListMax = 15;
	int initCost;
	int addCost;
	
	QStringList shellExt;
	QStringList textExt;
	QString memoAlter;
	QStringList recent;
	int maxRecent;

	bool f1focused;

	void makeMenu();
	void loadCfg();
	void saveCfg();
	void reloadCfg();
	void setScrn();
	void buildMenu();
	FileView* currentView();
	FileView* primaryView();
	FileView* secondaryView();
	QValueList<QFileInfo> doCompare(int op, QValueList<QFileInfo>& f1,QValueList<QFileInfo>& f2) const;
//	QStringList doCompare(int op, QValueList<QFileInfo>& f1,QValueList<QFileInfo>& f2) const;
	void setMenuEnable(QPopupMenu* current);
	bool setArg(int i , QStringList& p1,QStringList& p2);
	QStringList expandFile(int type,FileView* file,const QString& cap);
	void reloadView(FileItem* fi);
	void setUseFile(const QFileInfo& f);
	QStringList evalMark();
	void setStBar();
	void setScrollBar(bool b);
	QString menuName(MenuItem* m);
	void setSize();
	void addRecent(QString s);

	inline void setMenu(QPopupMenu* menu, int i) {
		menu->insertItem( menuName(items.at(i)) , this , SLOT(menuExec(int)) ,MenuItem::hkeyCode(items.at(i)->hotKey()),i);
	};
	inline void setMenuItem(QPopupMenu* menu,const char* slot,int i) {
		menu->insertItem( menuName(items.at(i)), this, slot,MenuItem::hkeyCode(items.at(i)->hotKey()));
	};
	inline void setMenuItemId(QPopupMenu* menu,const char* slot,int i,int id) {
		menu->insertItem( menuName(items.at(i)), this, slot,MenuItem::hkeyCode(items.at(i)->hotKey()),id);
	};
	inline void setMenuMenu(QPopupMenu* menu,QPopupMenu* subMenu,int i) {
		menu->insertItem( menuName(items.at(i)), subMenu);
	};

	inline int file1Width(int i) {
		return (QApplication::desktop()->width() / 8) * ( i + 2);
	};

	inline int file2Width(int i) {
		return (QApplication::desktop()->width() / 8) * ( 8- (i + 2));
	};
	inline int file1Height(int i) {
		return (QApplication::desktop()->height() / 8) * (i + 2);
	};
	inline int file2Height(int i) {
		return (QApplication::desktop()->height() / 8) * (8-(i + 2));
	};

	enum ViewMenuItem {
		Reload,
		Filter,
		Compare,
		SelectAll,
		SwapView,
		Property,
		SelectInvert,
		Disp2nd,
		MatchDirectory,
		StatusBar,
		ScrollBar
	};

	enum OptMenuItem {
		Preview,
		Resize,
		Direction,
		MyZaurus,
		Hidden,
		AddMarkManual,
		CallMark,
		Option
	};

	enum SizeMenuItem {
		Horizontal,
		SplitBase
	};

	static const int SplitMax = 4;

};

