#include <qdir.h>
#include <qstring.h>
#include <qpe/mimetype.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qmessagebox.h>
#include <qpe/qpeapplication.h>
#include <sys/vfs.h>
#include <qstack.h>
#include <qpe/config.h>
#include <qpainter.h>
#include <qbrush.h>

#include "FLib.h"
#include "FileView.h"

FileView::FileView( const QString &path , const QString &nameFilter, bool dicon, bool hidden, bool myZau ,bool dispF , QWidget * parent , const char * name  , WFlags f  ) : QListView(parent, name, f){

	sizeX = 320;
	sizeY = 100;
	vga = FLib::isVga();
	addColumn(tr("FileName")) ;
	addColumn(tr("Size"));
	addColumn(tr("Date"));
	setColumnWidthMode(FileName,QListView::Manual);
	setColumnWidthMode(FileSize,QListView::Manual);
	setColumnWidthMode(FileDate,QListView::Manual);
	setAllColumnsShowFocus(TRUE);
	setShowSortIndicator(TRUE);
	setSorting(FileName);
	setColumnAlignment(FileSize,Qt::AlignRight);
	setColumnAlignment(FileDate,Qt::AlignRight);

	setSelectionMode(QListView::Extended);

	nFilter = nameFilter;
	hiddenFile = hidden;
	myZaurus = myZau;
	dispFile = dispF;
	dispIcon = dicon;
	baseD = "";
	loadCfg();
	setCurrentFile(path);

	connect( this , SIGNAL(returnPressed(QListViewItem *)) , this , SLOT(selectItem(QListViewItem *)) );
	connect( this , SIGNAL(doubleClicked(QListViewItem *)) , this , SLOT(selectItem(QListViewItem *)) );
	connect( this, SIGNAL(selectionChanged()) ,this , SLOT(selectChanged() ));

}

FileView::~FileView()
{
}

bool FileView::getDirInfo(const QString &path)
{
qDebug("getDirInfo:: path: %s",(const char *)path);
	dirType = NormalDir;
	selCount = -1;
	if (myZaurus && (path == FileItem::pathMyZaurus() || path == ""
			|| path == "/mnt" || path == "/var/mnt" || path == "/usr/mnt.rom"
			|| path == "/home/zaurus" ) ){
		QString s;
		if (columns() > FilePath) {
			removeColumn(FilePath);
		}
		clear();
		liType = Normal;
		QFileInfo fi(FileItem::pathMain_Memory());
		new FileItem(this,fi);
		if (FileItem::availSD_Card()) {
			fi.setFile(FileItem::pathSD_Card());
			new FileItem(this,fi);
		}
		if (FileItem::availCF_Card()) {
			fi.setFile(FileItem::pathCF_Card());
			new FileItem(this,fi);
		}
		currentPath= FileItem::pathMyZaurus();
		setCurrentItem(firstChild());
		return FALSE;
	}
	QDir d(path,nFilter);
	int filter = QDir::Dirs;
	if (dispFile ) {
		filter  |= QDir::Files;
	}
	if ( hiddenFile ) {
		filter |= QDir::Hidden;
	}
	d.setFilter( filter );
	d.setMatchAllDirs(TRUE);
	const QFileInfoList *list = d.entryInfoList();
	if (list == NULL) {
		return TRUE;
	}
	currentPath= d.path();
	if ( customBack ) {
		const QString cpath = d.canonicalPath();
		if ( cpath.left(17) == "/usr/mnt.rom/card" ) {
			dirType = SdDir;
		} else if ( cpath.left(15) == "/usr/mnt.rom/cf") {
			dirType = CfDir;
		}
	}
	QFileInfoListIterator it( *list );
	QFileInfo *fi;  
	if (columns() > FilePath) {
		removeColumn(FilePath);
	}
	clear();
	liType = Normal;
	bool dd = false;
	while ( (fi=it.current()) ) {
		if (fi->fileName() != ".") {
			new FileItem(this,(*fi));
		}
		if (fi->fileName() == "..") {
			dd = true;
		}
		++it;
	}
	if ( ! dd ) {
		new FileItem(this, QFileInfo(d.path() + "/.."));
	}
	setCurrentItem(firstChild());
	return FALSE;
}

void FileView::selectItem(QListViewItem *it)
{
	if (it == 0 ) {
		return;
	}
	FileItem *item = (FileItem *)it;
	if (item->info()->isDir()) {
		changeDir(item->info());
	} else {
		emit fileSelected((*item->info()));
	}
}

void FileView::changeDir(const QFileInfo* f)
{
	if (myZaurus) {
		QString fp = f->fileName();
		if (currentPath == FileItem::pathMyZaurus()) {
			if (fp == FileItem::dirMain_Memory() ) {
				getDirInfo(FileItem::pathMain_Memory());
			} else if (fp == FileItem::dirSD_Card()) {
				getDirInfo(FileItem::pathSD_Card());
			} else if (fp == FileItem::dirCF_Card() ) {
				getDirInfo(FileItem::pathCF_Card());
			} else {
				return;
			}
		} else {
			if ( (FileItem::isMain_Memory(currentPath) || 
				FileItem::isSD_Card(currentPath) ||
				FileItem::isCF_Card(currentPath) ) && 
				(fp == "..") ){
				getDirInfo(FileItem::pathMyZaurus());
			} else {
				if (fp == "..") {
					setCurrentFile(currentPath+"");
				} else {
					getDirInfo(currentPath+"/"+fp);
				}
			}
		}
		setSelected(currentItem(),true);
	} else {
		if (f->fileName() == "..") {
			setCurrentFile(currentPath+"");
		} else {
			setCurrentFile(f->filePath() + "/");
		}
	}
	ensureItemVisible(currentItem());
	const QFileInfo* fi = ((FileItem *)currentItem())->info();
	emit selectionChanged(*fi);
	emit dirChanged();
}

void FileView::selectChanged()
{
	selCount = -1;
	FileItem *item = (FileItem*)currentItem();
	emit selectionChanged((*item->info()));
}

const QFileInfo *FileView::selected()
{
	FileItem *item =(FileItem *)currentItem();
	return item->info();
}


void FileView::updateView()
{
	getDirInfo(currentPath+"");
	setCurrentItem(firstChild());
	setSelected(currentItem(),true);
	ensureItemVisible(currentItem());
}

const QString FileView::nextItemName()
{
	QListViewItem* item = currentItem();
	QListViewItem* nextItem = 0;

	if (item->itemBelow() != 0 ) {
		nextItem = item->itemBelow();
		while ( nextItem != 0 && nextItem->isSelected() ) {
			item = nextItem;
			nextItem = item->itemBelow();
		}
	}
	if ( nextItem == 0) {
		nextItem = item->itemAbove();
		while ( nextItem->isSelected() ) {
			item = nextItem;
			nextItem = item->itemAbove();
			if (nextItem == 0 ) {
				return QString::null;
			}
		}
	}
	return ((FileItem*)nextItem)->info()->fileName();
}

void FileView::setCurrentFile(const QString &file)
{
	QFileInfo f(file);
	getDirInfo(f.dirPath());
	QListViewItemIterator it(this);
	for ( ; it.current() ; ++it) {
		if ( ((FileItem*)it.current())->info()->filePath() == file) {
			setCurrentItem(it.current());
			break;
		}
	}
	setSelected(currentItem(),true);
	ensureItemVisible(currentItem());
}


void FileView::keyPressEvent ( QKeyEvent *e )
{
	if (e->key() == Qt::Key_BackSpace) {
		changeDir(((FileItem*)firstChild())->info());
	} else {
		QListView::keyPressEvent(e);
	}
}

void FileView::setHidden(bool b)
{
	hiddenFile = b;
	updateView();
}

void FileView::setMyZaurus(bool b)
{
	myZaurus = b;
	if (myZaurus) {
		getDirInfo(FileItem::pathMyZaurus());
		setSelected(currentItem(),true);
		ensureItemVisible(currentItem());
	} else {
		if (currentPath== FileItem::pathMyZaurus()) {
			setCurrentFile(FileItem::pathMain_Memory());
		}
		updateView();
	}
}

bool FileView::getMyZaurus()
{
	return myZaurus;
}

const QString FileView::currentViewPath()
{
	if ( liType == Selection) {
		return selTypeTitle;
	}
	return currentDirName();
}

const QString FileView::currentDirName()
{
	if (! myZaurus) {
		return currentPath;
	} else {
		if (currentPath == FileItem::pathMyZaurus()) {
			return currentPath;
		} else {
			QString r = currentPath;
			r.replace(QRegExp("^"+FileItem::pathMain_Memory()),tr("Main_Memory"));
			r.replace(QRegExp("^"+FileItem::pathSD_Card()),tr("SD_Card"));
			r.replace(QRegExp("^"+FileItem::pathCF_Card()),tr("CF_Card"));
			return r;
		}
	}
}

const QString FileView::currentPathName()
{
	return currentPath;
}

void FileView::setNameFilter(const QString &f)
{
	nFilter = f;
}
const QString FileView::currentFilter()
{
	return nFilter;
}

void FileView::resizeEvent(QResizeEvent *e)
{
	if (e) {
		QListView::resizeEvent(e);
		setColumnSize();
	}
}
void FileView::focusInEvent(QFocusEvent *e)
{
	if (e) {
		emit hasFocused();
		QListView::focusInEvent(e);
	}
}

void FileView::setColumnSize()
{
	int fs = fontInfo().pointSize();
qDebug("font size = %d",fs);
	int ws = width();
	int ds = 7 * (fs+2);
	int ss = 3 * (fs+2);
	int ns = ws - (ds+ss);
	if (vga) {
		ns -= 36;
	} else {
		ns -= 18;
	}
	if (liType == Selection) {
		ns -= fs * 8;
	}
	if (ns - (10 *(fs+2)) < 0 ) {
		ns = 10 *(fs+2);
	}
	setColumnWidth(FileName, ns);
	setColumnWidth(FileSize, ss);
	setColumnWidth(FileDate, ds);
}

void FileView::getSelected()
{
	selFiles.clear();
	selCount = 0;
	sizeTotal = 0;
	int dir = 0;
	int file = 0;
	QListViewItemIterator it( this );
	for ( ; it.current(); ++it ) {
		FileItem* item = (FileItem*) it.current();
		if ( item->isSelected() ) {
			selFiles.append(*(item->info()) );
			++selCount;
			sizeTotal += item->info()->size();
			if (item->info()->isDir() ) {
				++dir;
			}
			if (item->info()->isFile() ) {
				++file;
			}
		}
	}
	isDirFlag = (dir == selCount);
	isFileFlag = ( selCount >0 && file == selCount);
	hasDirFlag = (dir >0 );
}

QValueList<QFileInfo>& FileView::multiSelected()
{
	if (selCount < 0 ) {
		getSelected();
	}
	return selFiles;
}

bool FileView::selIsDir()
{
	if (selCount < 0 ) {
		getSelected();
	}
	return isDirFlag;
}

bool FileView::selIsFile()
{
	if (selCount < 0 ) {
		getSelected();
	}
	return isFileFlag;
}

bool FileView::selHasDir()
{
	if (selCount < 0 ) {
		getSelected();
	}
	return hasDirFlag;
}

int FileView::selFileCount()
{
	if (selCount < 0 ) {
		getSelected();
	}
	return selCount;
}

QValueList<QFileInfo> FileView::allFiles(bool d)		// true - file only : false - includes dir
{
	QValueList<QFileInfo> files;
	QListViewItemIterator it( this );
	for ( ; it.current(); ++it ) {
		FileItem* item = (FileItem*) it.current();
		if (item->text(FileName) != "..") {
			if (!d || ! item->info()->isDir() ) {
				files.append(*(item->info()) );
			}
		}
	}
	return files;
}

const QString FileView::selectedMsg()
{
	if (selCount < 0 ) {
		getSelected();
	}
	if (selCount == 0
		|| (selCount == 1 && selIsDir() )  ) {
		struct statfs stat;
		if (statfs((const char *)currentPath, &stat) == 0 ) {
			return tr("%1 files : Free  ").arg(childCount(),3) + displayBytes(stat.f_bavail*stat.f_bsize);
		} else {
			return QString::null;
		}
	} else {
		return tr("%1 files : Total ").arg(selCount,3) + displayBytes(sizeTotal);
	}
}

const QString FileView::displayBytes(int bytes) 
{
	if (bytes > 99999999 ) {
		return FLib::formatNum(bytes / 1024,11) +"K bytes" ;
	}
	return FLib::formatNum(bytes,12) + " bytes";
}

void FileView::selAll()
{
	clearSelection();
	QListViewItemIterator it( this );
	for ( ; it.current(); ++it ) {
		FileItem* item = (FileItem*) it.current();
		if (item->info()->isFile() ) {
			item->setSelected(TRUE);
		}
	}
	selCount = -1;
	triggerUpdate();
	FileItem* item = (FileItem*) currentItem();
	emit selectionChanged((*item->info()));
}

void FileView::fileSpecify(QValueList<QFileInfo>& fileList)
{
	if (fileList.count() == 0 ) {
		return;
	}
	QListViewItemIterator it( this );
	for ( ; it.current(); ++it ) {
		FileItem* item = (FileItem*)it.current();
		QValueList<QFileInfo>::Iterator li = fileList.begin();
		for ( ; li != fileList.end() ; ++ li) {
			if ( FLib::matchFileInfo(*(li) , *(item->info()) ) ) {
				break;
			}
		}
		if (li != fileList.end()) {
			it.current()->setSelected(TRUE);
			fileList.remove(li);
			if (fileList.count() == 0 ) {
				break;
			}
		}
	}
	selCount = -1;
	triggerUpdate();
}

void FileView::selectInvert()
{
	QListViewItemIterator it( this );
	bool hasDir = selHasDir();
	++it;
	for ( ; it.current(); ++it ) {
		FileItem* item = (FileItem*) it.current();
		if (!item->isSelected() && ( item->info()->isFile() || hasDir) ){
			item->setSelected(TRUE);
		} else {
			item->setSelected(FALSE);
		}
	}
	selCount = -1;
	triggerUpdate();
}

void FileView::setFiles(const QString& title, const QString& base, QStringList& flist)
{
	dirType = NormalDir;
	liType = Selection;
	selTypeTitle = title;
	baseD = base;
	clear();
	if (columns() <= FilePath) {
		int fs = fontInfo().pointSize();
		addColumn(tr("FilePath"),fs * 20);
	}
	QFileInfo d("..");
	new FileItem(this,d);
	for (QStringList::Iterator it = flist.begin() ; it != flist.end() ; ++ it) {
		QFileInfo fi(*it);
		if (fi.exists()) {
			new FileItem(this,fi);
		}
	}
	setCurrentItem(firstChild());
	setSelected(currentItem(),true);
	ensureItemVisible(currentItem());
}

void FileView::reloadView(FileItem* fi)
{
	if (liType == Selection) {
		QListViewItemIterator it(this);
		bool b = false;
		{
			QStack<FileItem> del;
			del.setAutoDelete(true);
			for ( ; it.current() ; ++it) {
				FileItem* cur = (FileItem*)it.current();
				if ( fi == cur ) {
					b = true;
				}
				if (cur->info()->exists()) {
					if (! b) {
						setCurrentItem(it.current());
					}
				}  else {
					del.push(cur);
				}
			}
		}
		setSelected(currentItem(),true);
		return;
	}
	if (fi == NULL) {
		updateView();
	} else if (fi->info()->exists() ) {
		setCurrentFile(fi->info()->filePath());
	} else {
		setCurrentFile(fi->info()->dirPath() + "/" + nextItemName());
	}
}

QSize FileView::sizeHint() const
{
	QSize s(sizeX,sizeY);
	return s;
}

void FileView::paintEmptyArea ( QPainter * p, const QRect & rect) 
{
	if ( customBack ) {
		const QBrush b(backColor());
		p->fillRect( rect, b);
	} else {
		QListView::paintEmptyArea(p,rect);
	}
}

QColor FileView::backColor()
{
	switch ( dirType ) {
	case SdDir:
		return sBack;
	case CfDir:
		return cBack;
	default:
		return nBack;
	}
}
void FileView::setSize(int x,int y)
{
	sizeX = x;
	sizeY = y;
}

QString FileView::dropBase(const QFileInfo& f)
{
	QString node = f.filePath().mid(baseD.length());
	if (node.left(1) == "/") {
		node = node.mid(1);
	}
	return node;
}

void FileView::loadCfg()
{
	int r;
	int g;
	int b;
	Config cf = Config("FileLaunch");
	cf.setGroup("color");
	customBack = cf.readBoolEntry("UseCustomBack",false);
	r = cf.readNumEntry("NormalLinkR",0);
	g = cf.readNumEntry("NormalLinkG",0);
	b = cf.readNumEntry("NormalLinkB",255);
	nLink.setRgb(r,g,b);
	r = cf.readNumEntry("CardLinkR",0);
	g = cf.readNumEntry("CardLinkG",0);
	b = cf.readNumEntry("CardLinkB",255);
	cLink.setRgb(r,g,b);
	r = cf.readNumEntry("NormalBackR",255);
	g = cf.readNumEntry("NormalBackG",255);
	b = cf.readNumEntry("NormalBackB",255);
	nBack.setRgb(r,g,b);
	r = cf.readNumEntry("SdBackR",255);
	g = cf.readNumEntry("SdBackG",255);
	b = cf.readNumEntry("SdBackB",255);
	sBack.setRgb(r,g,b);
	r = cf.readNumEntry("CfBackR",255);
	g = cf.readNumEntry("CfBackG",255);
	b = cf.readNumEntry("CfBackB",255);
	cBack.setRgb(r,g,b);

}
/*

*/

FileItem::FileItem( QListView * parent , QFileInfo itemInfo ) : QListViewItem( parent)
{
	fileInfo = itemInfo;
	QString fn = fileInfo.fileName();
	QString fp = fileInfo.filePath();
	FileView* mom = (FileView*)parent;
	if (mom->listType() == FileView::Selection  && fn == ".." ) {
		fileInfo = QFileInfo(mom->baseDir());
	}
	myZaurus = mom->getMyZaurus();
	MimeType mt(fp);
	bool dispIcon = mom->dispIconF();
	if ( fn == "" ) {
		fn = ".";
	}
	if (! dispIcon && fileInfo.isDir()) {
		setText(FileView::FileName , fn + "/");
	} else {
		setText(FileView::FileName,fn);
	}
	if (dispIcon ) {
		QPixmap pm;
		if (fileInfo.isDir()) {
			if (fn == "..") {
				pm = Resource::loadPixmap( "folderup" );
			} else if (isMain_Memory(fp)) {
				pm = Resource::loadPixmap( "MainDeviceS" );
				if (myZaurus) {
					setText(FileView::FileName,QObject::tr("Main_Memory"));
				}
			} else if (isSD_Card(fp)) {
				pm = Resource::loadPixmap( "SDDeviceS" );
				if (myZaurus) {
					setText(FileView::FileName,QObject::tr("SD_Card"));
				}
			} else if (isCF_Card(fp)) {
				pm = Resource::loadPixmap( "CFDeviceS" );
				if (myZaurus) {
					setText(FileView::FileName,QObject::tr("CF_Card"));
				}
			} else {
				pm = Resource::loadPixmap( "slfolder" );
			}
		} else {
			pm = mt.pixmap();
		}
		if (pm.isNull()) {
			pm = Resource::loadPixmap("slUnknown14");
		}
		setPixmap(FileView::FileName,pm);
	}
	if (!fileInfo.isDir()) {
		QString sz = fileInfo.size() > 1024 ? 
			QString::number(fileInfo.size()/1024)+ "K" :
			QString::number(fileInfo.size());
		setText(FileView::FileSize,sz );
	}
	QString dd ;
	QDate d = fileInfo.lastModified().date();
	QTime t = fileInfo.lastModified().time();
	dd.sprintf("%02d/%02d/%02d %02d:%02d",d.year()%100,d.month(),d.day(),t.hour(),t.minute());
	setText(FileView::FileDate,dd);
	if (mom->columns() > FileView::FilePath) {
		setText(FileView::FilePath," "+fileInfo.dirPath());
	}
}

FileItem::~FileItem()
{
}
const QFileInfo* FileItem::info()
{
	return &fileInfo;
}

QString FileItem::key( int column, bool ascending ) const
{
	if (text(FileView::FileName) == "..") {
		if (ascending) {
			return " ";
		} else {
			return QChar(0xffffU);
		}
	} else if( (column == FileView::FileName) && fileInfo.isDir() ){ 
		if ( myZaurus  && isMain_Memory(fileInfo.filePath())  ){ 
			return "!!"+ text(column).lower();
		} else if ( myZaurus  && isSD_Card(fileInfo.filePath())  ){ 
			return "!#"+ text(column).lower();
		} else { 
			return "!"+ text(column).lower();
		}
	} else if ( column == FileView::FileSize) {
		return QString("%1").arg(fileInfo.size(),20);
	} else {
		return text(column).lower();
	}
}

bool FileItem::isMain_Memory(const QString &fp)
{
	return (fp +"/" == QPEApplication::documentDir()) ;
}
bool FileItem::isSD_Card(const QString &fp)
{
	return (fp == "/mnt/card" || fp == "/var/mnt/card" || fp == "/usr/mnt.rom/card" );
}

bool FileItem::isCF_Card(const QString &fp)
{
	return (fp == "/mnt/cf" || fp == "/var/mnt/cf" || fp == "/usr/mnt.rom/cf" );
}

bool FileItem::availSD_Card()
{
	QFileInfo f(pathSD_Card()+"/NotAvailable");
	if (f.exists() && (f.size() == 0) && !f.isDir()) {
		return FALSE;
	}
	return TRUE;
}

bool FileItem::availCF_Card()
{
	QFileInfo f(pathCF_Card()+"/NotAvailable");
	if (f.exists() && (f.size() == 0) && !f.isDir()) {
		return FALSE;
	}
	return TRUE;
}
const QString FileItem::pathMain_Memory()
{
	QString s = QPEApplication::documentDir();
	return s.left(s.length()-1);
}

void FileItem::paintCell ( QPainter * p, const QColorGroup & cg, int column, int width, int align )
{
	QColorGroup c(cg);
	if (info()->isSymLink() ) {
		struct statfs stat;
		if (statfs(info()->filePath().utf8(), &stat) == 0 ) {
			switch ( stat.f_type ) {
			case 0xEF51:
			case 0x4d44:
			case 0x28cd3d45:
				c.setColor(QColorGroup::Text, ((FileView*)listView())->cardLink());
				break;
			default:
				c.setColor(QColorGroup::Text, ((FileView*)listView())->normalLink());
			}
		} else {
			c.setColor(QColorGroup::Text, ((FileView*)listView())->normalLink());
		}
	}
	if ( ((FileView*)listView())->useCustomBack()) {
		c.setColor(QColorGroup::Base, ((FileView*)listView())->backColor());
	}
	QListViewItem::paintCell(p,c,column,width,align);
}
