#include <qlistview.h>
#include <qfileinfo.h>
#include <qvaluelist.h>
#include <qsize.h>
#include <qcolor.h>

class FileItem;

class FileView : public QListView
{
	Q_OBJECT

public:
	FileView( const QString &path , const QString &nameFilter = "*", bool dicon = TRUE ,bool hidden = FALSE, bool myZau = FALSE, bool dispF = TRUE, QWidget * parent = 0, const char * name = 0 , WFlags f = 0 );
	~FileView() ;
	const QFileInfo* selected();
	QValueList<QFileInfo>& multiSelected();
	QValueList<QFileInfo> allFiles(bool d);
	bool selIsDir();
	bool selIsFile();
	bool selHasDir();
	int selFileCount();
	void selectInvert();
	void updateView();
	const QString nextItemName();
	void setCurrentFile(const QString &file);
	void setHidden(bool b);
	void setMyZaurus(bool b);
	bool getMyZaurus();
	void setNameFilter(const QString &filter);
	const QString currentDirName();
	const QString currentViewPath();
	const QString currentPathName();
	void selAll();
	void fileSpecify(QValueList<QFileInfo>& fileList);
//	void fileSpecify(QStringList& fileList);
	const QString currentFilter();

	void setFiles(const QString&, const QString &, QStringList&);
	void reloadView(FileItem* fi);

	virtual QSize sizeHint() const;

	void setSize(int x, int y);

	const QString selectedMsg();
	bool dispIconF() {  return dispIcon; };
	int listType() { return liType; };
	const QString baseDir() { return baseD;};
	QString dropBase(const QFileInfo& f);

	void loadCfg();
	
	QColor normalLink() const { return nLink; };
	QColor cardLink()   const { return cLink; };

	bool useCustomBack() const { return customBack; };
	QColor backColor();
	

	enum ColumnName {
		FileName,
		FileSize,
		FileDate,
		FilePath
	};
	enum ListType {
		Normal,
		Selection
	};
	

signals:
	void fileSelected(const QFileInfo &);
	void selectionChanged(const QFileInfo &);
	void dirChanged();
	void hasFocused();

protected:
	virtual void keyPressEvent ( QKeyEvent *e );
	virtual void resizeEvent(QResizeEvent *e);
	virtual void focusInEvent(QFocusEvent *e);
	virtual void paintEmptyArea ( QPainter *, const QRect & ) ;

private slots:
	void selectItem(QListViewItem *);
	void selectChanged();

private:
	bool getDirInfo(const QString &path);  // if err return true
	void changeDir(const QFileInfo* f);
	void setColumnSize();
	bool removeFile(const QString &fp,bool isDir);
	void getSelected();
	const QString displayBytes(int bytes);

	QString currentPath;
	QString nFilter;
	bool hiddenFile;
	bool myZaurus;
	bool vga;
	bool dispFile;
	int selCount;
	bool isDirFlag;
	bool isFileFlag;
	bool hasDirFlag;
	int sizeTotal;
	QValueList<QFileInfo> selFiles;
	int liType;
	QString selTypeTitle;
	QString baseD;

	bool dispIcon;
	
	int sizeX;
	int sizeY;

	QColor nLink;
	QColor cLink;
	bool customBack;
	QColor nBack;
	QColor sBack;
	QColor cBack;
	int dirType;
	enum DirType {
		NormalDir,
		SdDir,
		CfDir
	};
};

class FileItem : public QListViewItem
{
//	Q_OBJECT

public:
	FileItem( QListView * parent ,const QFileInfo itemInfo);
	~FileItem();
	const QFileInfo *info();
	virtual QString key( int column, bool ascending = TRUE ) const;
	virtual void paintCell ( QPainter *, const QColorGroup & cg, int column, int width, int alignment ) ;
	static bool isMain_Memory(const QString &fp);
	static bool isSD_Card(const QString &fp);
	static bool isCF_Card(const QString &fp);
	static bool availSD_Card();
	static bool availCF_Card();
	static inline const QString pathMyZaurus() { return "My Zaurus";};
	static const QString pathMain_Memory();
	static inline const QString pathSD_Card() { return "/usr/mnt.rom/card"; };
	static inline const QString pathCF_Card() { return "/usr/mnt.rom/cf"; };
	static inline const QString dirMain_Memory() { return "Documents"; };
	static inline const QString dirSD_Card() { return "card"; };
	static inline const QString dirCF_Card() { return "cf"; };

private:
	QFileInfo fileInfo;
	bool myZaurus;
};
