#include "ProcDialog.h"
#include <qstringlist.h>
#include <qtextcodec.h>

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <sys/time.h>

#include "../shared/MiscLib.h"
#include "FLib.h"

extern int errno;

static ProcDialog* procdialog;


/* ------------------------------------------------------------------------ */
/*	from shellcommander ProcessInvoker.cpp 									*/
/* ------------------------------------------------------------------------ */

static Sigfunc* setSignalHandler(int signo, Sigfunc* handler)
{
	struct sigaction act,oact;

	act.sa_handler = handler;
	::sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	if(::sigaction(signo, &act, &oact) < 0){
		return(NULL);
	}
	return(oact.sa_handler);
}

static void childHandler(int /*signo*/)
{
	pid_t pid;
	int status;
	while((pid = ::waitpid(-1, &status, WNOHANG)) > 0){
		if(pid == procdialog->child() && procdialog != NULL){
			procdialog->procTerm(status);
		}
	}
}


ProcDialog::ProcDialog(const QString& cmd, const QString& pa, int fontSize,QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
	procdialog = this;
	setCaption( tr( "Processing Command" ) );
	termEntered = false;
	layout = new QGridLayout(this);

	view = new View(this);
	if (FLib::isVga() ) {
		view->setFont(QFont("lcfont",fontSize));
	}
	layout->addMultiCellWidget(view,0,0,0,2);

	sel = new QPushButton(this);
	sel->setText(tr("SelAll"));
	sel->setAccel(CTRL+Key_A);
	layout->addMultiCellWidget(sel,1,1,0,0);

	can = new QPushButton(this);
	can->setText(tr("KILL"));
	layout->addMultiCellWidget(can,1,1,1,1);

	cp = new QPushButton(this);
	cp->setText(tr("Copy"));
	cp->setAccel(CTRL+Key_C);
	layout->addMultiCellWidget(cp,1,1,2,2);

	cmdName = cmd;
	param = pa;
	lastStr[ReadOut] = QCString(0);
	lastStr[ReadErr] = QCString(0);

	can->setFocus();
	connect( can , SIGNAL(clicked())   , this , SLOT(pressKill()) );
	connect( sel,  SIGNAL(clicked())   , view , SLOT(selectAll()) );
	connect( cp ,  SIGNAL(clicked())   , view , SLOT(copy())  );
	termed = FALSE;
	showMaximized();


}

ProcDialog::~ProcDialog()
{
	procdialog = NULL;
}

void ProcDialog::showEvent ( QShowEvent *e )
{
	QDialog::showEvent(e);
	procStart();
}

void ProcDialog::procStart()
{
	QStringList arg;
	arg.append("/bin/bash");
	arg.append("-c");
	arg.append("\"" + cmdName + "\" " + param);
	if (::pipe(stdOut) < 0 ) {
		view->insertLine(tr("pipe stdout open error"));
		return;
	}
	if (::pipe(stdErr) < 0 ) {
		view->insertLine(tr("pipe stderr open error"));
		return;
	}

	defChildHandler = setSignalHandler(SIGCHLD, SIG_DFL);

	if ((childId = ::fork()) < 0 ) {
		view->insertLine(tr("fork error"));
		setSignalHandler(SIGCHLD, defChildHandler);
		return;
	} else if (childId == 0 ) {
		::close(stdOut[0]);
		::close(stdErr[0]);
		::close(STDOUT_FILENO);
		::dup2(stdOut[1],STDOUT_FILENO);
		::close(stdOut[1]);
		::close(STDERR_FILENO);
		::dup2(stdErr[1],STDERR_FILENO);
		::close(stdErr[1]);
		::setpgid(0,getpid());
		QCString* arglist = new QCString[arg.count()+1];
		const char** argv = new const char*[arg.count()+1];
		unsigned int i;
		for(i=0; i<arg.count(); i++){
			arglist[i] = arg[i].utf8();
			argv[i] = arglist[i];
		}
		argv[i] = 0;
		::execv(argv[0], (char*const*)argv);
		delete[] arglist;
		delete[] argv;
		::_exit(127);
	}
	::close(stdOut[1]);
	::close(stdErr[1]);
	setpgid(childId,childId);
	can->setText(tr("KILL")+ QString("(%1)").arg(childId));
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()) , this , SLOT(procCheck()) ) ;
	int w;
	int pid = ::waitpid(childId,&w,WNOHANG);
	if (pid > 0 ) {
		procTerm(w);
	} else if (pid == -1) {
		qDebug("error, processed pid = -1 ");
	} else {
		setSignalHandler(SIGCHLD, childHandler);
		timer->start(ProcCheckInterval);
	}
}

void ProcDialog::procCheck()
{
	int w;
	int pid = ::waitpid(childId,&w,WNOHANG);
	if (pid == 0 ) {
		readPipe(ReadOut,false);
		readPipe(ReadErr,false);
	} else {
		readPipe(ReadOut,true);
		readPipe(ReadErr,true);
		timer->stop();
		if (view->numLines() <= 2 && result == 0) {
			QDialog::accept();
		}
	}
}

void ProcDialog::pressKill()
{
	if (termEntered && ! termed) {
		::killpg(childId, SIGKILL);
	} else if (! termed) {
		::killpg(childId, SIGTERM);
		termEntered = true;
	} else {
		QDialog::accept();
	}
}

bool ProcDialog::checkPipe(int t)
{
	int fd = (t == ReadOut) ? stdOut[0] : stdErr[0];
	struct timeval timeOut;
	timeOut.tv_sec = 0;
	timeOut.tv_usec = 0;

	fd_set fdset;
	FD_ZERO(&fdset);
	FD_SET(fd , &fdset);

	int r = ::select(FD_SETSIZE, &fdset, NULL, NULL, &timeOut);
	if (r > 0 && FD_ISSET(fd, &fdset)) {
		return true;
	}
	return false;
}

void ProcDialog::readPipe(int t,bool last)
{
	int fd = (t == ReadOut) ? stdOut[0] : stdErr[0];

	char buf[PIPEBUF+1];
	QCString str(lastStr[t]);
	
	while ( checkPipe(t)) {
		int rsize  = ::read(fd,buf,PIPEBUF);
		buf[rsize] = 0;
		if (rsize == 0 ) {
			break;
		}
		const char *rbuf = buf;
		str += rbuf;
	}

	if (str.length() == 0 ) {
		return;
	}
	int nl = str.findRev('\n');
	if (nl >=0) {
		lastStr[t] = str.mid(nl+1);
		str.truncate(nl);
	} else if (! last ) {
		return;
	}

	QTextCodec *codec = QTextCodec::codecForName(MiscLib::setCode((const char *)str,str.length()));

	if (t == ReadOut) {
		view->insertLine(codec->toUnicode(str,str.length()));
		view->setCursorPosition(view->numLines(),0);
	} else {
		view->insertLine(codec->toUnicode(str,str.length()));
		view->setCursorPosition(view->numLines(),0);
	}
}

void ProcDialog::procTerm(int w)
{
	setSignalHandler(SIGCHLD, defChildHandler);
	if (WIFEXITED(w)) {
		result = WEXITSTATUS(w);
		if (result >= 125) {
			can->setText(tr("ERROR")+ QString("(%1)").arg(result));
		} else {
			can->setText(tr("TERMINATE %1").arg(result));
		}
	} else if (WIFSIGNALED(w)) {
		can->setText(tr("TERMINATE"));
		result = 0;
	} else if (WIFSTOPPED(w)){
		can->setText(tr("STOPPED"));
		result = 1;
	}
	termed = TRUE;
}

void ProcDialog::accept()
{
	if ( ! termed) {
		pressKill();
	} else {
		QDialog::accept();
	}
}

void ProcDialog::reject()
{
	if ( ! termed) {
		pressKill();
	} else {
		QDialog::reject();
	}
}
