#include <qdir.h>
#include <qstring.h>
#include <qpe/mimetype.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qmessagebox.h>
#include <qpe/qpeapplication.h>

#include "MiscLib.h"
#include "FileView.h"

FileView::FileView( const QString &path , const QString &nameFilter, bool hidden, bool myZau ,bool dispF , QWidget * parent , const char * name  , WFlags f  ) : QListView(parent, name, f){
	vga = MiscLib::isVga();
	addColumn(tr("FileName")) ;
	addColumn(tr("Size"));
	addColumn(tr("Date"));
	setColumnWidthMode(FileName,QListView::Manual);
	setColumnWidthMode(FileSize,QListView::Manual);
	setColumnWidthMode(FileDate,QListView::Manual);
	setAllColumnsShowFocus(TRUE);
	setShowSortIndicator(TRUE);
	setSorting(FileName);
	setColumnAlignment(FileSize,Qt::AlignRight);
	setColumnAlignment(FileDate,Qt::AlignRight);

	nFilter = nameFilter;
	hiddenFile = hidden;
	myZaurus = myZau;
	dispFile = dispF;

	setCurrentFile(path);

	connect( this , SIGNAL(returnPressed(QListViewItem *)) , this , SLOT(selectItem(QListViewItem *)) );
	connect( this , SIGNAL(clicked(QListViewItem *)) , this , SLOT(selectItem(QListViewItem *)) );
	connect( this, SIGNAL(selectionChanged(QListViewItem *)) ,this , SLOT(selectChanged(QListViewItem *) ));

}

FileView::~FileView()
{
}

bool FileView::getDirInfo(const QString &path)
{
qDebug("getDirInfo:: path: %s",(const char *)path);
	if (myZaurus && (path == FileItem::pathMyZaurus() || path == ""
			|| path == "/mnt" || path == "/var/mnt" || path == "/usr/mnt.rom"
			|| path == "/home/zaurus" ) ){
		QString s;
		clear();
		QFileInfo fi(FileItem::pathMain_Memory());
		new FileItem(this,fi);
		if (FileItem::availSD_Card()) {
			fi.setFile(FileItem::pathSD_Card());
			new FileItem(this,fi);
		}
		if (FileItem::availCF_Card()) {
			fi.setFile(FileItem::pathCF_Card());
			new FileItem(this,fi);
		}
		currentPath= FileItem::pathMyZaurus();
		setCurrentItem(firstChild());
		return FALSE;
	}
	QDir d(path,nFilter);
	int filter = QDir::Dirs;
	if (dispFile ) {
		filter  |= QDir::Files;
	}
	if ( hiddenFile ) {
		filter |= QDir::Hidden;
	}
	d.setFilter( filter );
	d.setMatchAllDirs(TRUE);
	const QFileInfoList *list = d.entryInfoList();
	if (list == NULL) {
		return TRUE;
	}
	currentPath= d.path();
	QFileInfoListIterator it( *list );  
	QFileInfo *fi;  
	clear();
	while ( (fi=it.current()) ) {
		if (fi->fileName() != ".") {
			new FileItem(this,(*fi));
		}
		++it;
	}
	setCurrentItem(firstChild());
	return FALSE;
}

void FileView::selectItem(QListViewItem *it)
{
	if (it == 0 ) {
		return;
	}
	FileItem *item = (FileItem *)it;
	if (item->info()->isDir()) {
		changeDir(item->info()->fileName());
	} else {
		emit fileSelected((*item->info()));
	}
}

void FileView::changeDir(const QString& fp)
{
	if (myZaurus) {
qDebug("fp %s",(const char *)fp);
qDebug("currentPath %s",(const char *)currentPath);
		if (currentPath == FileItem::pathMyZaurus()) {
			if (fp == FileItem::dirMain_Memory() ) {
				getDirInfo(FileItem::pathMain_Memory());
			} else if (fp == FileItem::dirSD_Card()) {
				getDirInfo(FileItem::pathSD_Card());
			} else if (fp == FileItem::dirCF_Card() ) {
				getDirInfo(FileItem::pathCF_Card());
			} else {
				return;
			}
		} else {
			if ( (FileItem::isMain_Memory(currentPath) || 
				FileItem::isSD_Card(currentPath) ||
				FileItem::isCF_Card(currentPath) ) && 
				(fp == "..") ){
				getDirInfo(FileItem::pathMyZaurus());
			} else {
				if (fp == "..") {
					setCurrentFile(currentPath+"");
				} else {
					getDirInfo(currentPath+"/"+fp);
				}
			}
		}
	} else {
		if (fp == "..") {
			setCurrentFile(currentPath+"");
		} else {
			getDirInfo(currentPath+"/"+fp);
		}
	}
	ensureItemVisible(currentItem());
	const QFileInfo* fi = ((FileItem *)currentItem())->info();
	emit selectionChanged(*fi);
	emit dirChanged();
}

void FileView::selectChanged(QListViewItem *it)
{
	FileItem *item = (FileItem *)it;
	emit selectionChanged((*item->info()));
}

const QFileInfo *FileView::selected()
{
	FileItem *item =(FileItem *)currentItem();
	return item->info();
}

void FileView::updateView()
{
	getDirInfo(currentPath+"");
	setCurrentItem(firstChild());
	ensureItemVisible(currentItem());
}

bool FileView::delFile (const QString &mymsg,bool forceF)
{
	FileItem* item = (FileItem *)currentItem();

	QString fp = item->info()->filePath();

	FileItem* nextItem;

	if (currentItem()->itemBelow() == 0 ) {
		nextItem = (FileItem* ) (currentItem()->itemAbove());
	} else {
		nextItem = (FileItem* ) (currentItem()->itemBelow());
	}
	QString nextName = nextItem->info()->filePath();

	if (forceF || QMessageBox::warning(this, tr("delete"),mymsg,tr("Yes"), tr("No"),QString::null,1)  == 0) {
//		if (QFile::remove(fp) ) {
		if (removeFile(fp,item->info()->isDir())) {
			setCurrentFile(nextName);
		} else {
			QMessageBox::warning(this, tr("delete"),tr("delete failed"));
		}
	}
	return TRUE;
}

bool FileView::removeFile(const QString &fp,bool isDir)
{
	if (isDir) {
		QDir d;
		return d.rmdir(fp);
	} else {
		return QFile::remove(fp);
	}
}

void FileView::setCurrentFile(const QString &file)
{
	QFileInfo f(file);
	getDirInfo(f.dirPath());
	QListViewItemIterator it(this);
	for ( ; it.current() ; ++it) {
		if ( ((FileItem*)it.current())->info()->filePath() == file) {
			setCurrentItem(it.current());
			break;
		}
	}
	ensureItemVisible(currentItem());
}


void FileView::keyPressEvent ( QKeyEvent *e )
{
	int ky = e->key();
	if (ky == Qt::Key_BackSpace) {
		changeDir("..");
	} else {
		QListView::keyPressEvent(e);
	}
}

void FileView::setHidden(bool b)
{
	hiddenFile = b;
	updateView();
}

void FileView::setMyZaurus(bool b)
{
	myZaurus = b;
	if (myZaurus) {
		getDirInfo(FileItem::pathMyZaurus());
		ensureItemVisible(currentItem());
	} else {
		if (currentPath== FileItem::pathMyZaurus()) {
			setCurrentFile(FileItem::pathMain_Memory());
		}
		updateView();
	}
}

bool FileView::getMyZaurus()
{
	return myZaurus;
}

const QString FileView::currentDirName()
{
	if (! myZaurus) {
		return currentPath;
	} else {
		if (currentPath == FileItem::pathMyZaurus()) {
			return currentPath;
		} else {
			QString r = currentPath;
			r.replace(QRegExp("^"+FileItem::pathMain_Memory()),tr("Main_Memory"));
			r.replace(QRegExp("^"+FileItem::pathSD_Card()),tr("SD_Card"));
			r.replace(QRegExp("^"+FileItem::pathCF_Card()),tr("CF_Card"));
			return r;
		}
	}
}

const QString FileView::currentPathName()
{
	return currentPath;
}

void FileView::setNameFilter(const QString &f)
{
	nFilter = f;
}

void FileView::resizeEvent(QResizeEvent *e)
{
	if (e) {
		QListView::resizeEvent(e);
		setColumnSize();
	}
}

void FileView::setColumnSize()
{
	int fs = fontInfo().pointSize();
qDebug("font size = %d",fs);
	int ws = width();
	int ds = 7 * (fs+2);
	int ss = 3 * (fs+2);
	int ns = ws - (ds+ss);
	if (vga) {
		ns -= 36;
	} else {
		ns -= 18;
	}
	if (ns - (10 *(fs+2)) < 0 ) {
		ns = 10 *(fs+2);
	}
	setColumnWidth(FileName, ns);
	setColumnWidth(FileSize, ss);
	setColumnWidth(FileDate, ds);
}

/*

*/

FileItem::FileItem( QListView * parent , QFileInfo itemInfo ) : QListViewItem( parent)
{
	fileInfo = itemInfo;
	QString fn = fileInfo.fileName();
	QString fp = fileInfo.filePath();
	myZaurus = ((FileView*)parent)->getMyZaurus();
	MimeType mt(fp);
	QPixmap pm;
	setText(FileView::FileName,fn);
	if (fileInfo.isDir()) {
		if (fn == "..") {
			pm = Resource::loadPixmap( "folderup" );
		} else if (isMain_Memory(fp)) {
			pm = Resource::loadPixmap( "MainDeviceS" );
			if (myZaurus) {
				setText(FileView::FileName,QObject::tr("Main_Memory"));
			}
		} else if (isSD_Card(fp)) {
			pm = Resource::loadPixmap( "SDDeviceS" );
			if (myZaurus) {
				setText(FileView::FileName,QObject::tr("SD_Card"));
			}
		} else if (isCF_Card(fp)) {
			pm = Resource::loadPixmap( "CFDeviceS" );
			if (myZaurus) {
				setText(FileView::FileName,QObject::tr("CF_Card"));
			}
		} else {
			pm = Resource::loadPixmap( "slfolder" );
		}
	} else {
		pm = mt.pixmap();
	}
	if (pm.isNull()) {
		pm = Resource::loadPixmap("slUnknown14");
	}
	setPixmap(FileView::FileName,pm);
	if (!fileInfo.isDir()) {
		QString sz = QString::number(fileInfo.size());
		if (sz.length() > 3) {
			sz = QString::number(fileInfo.size()/1000)+ "K";
		}
		setText(FileView::FileSize,sz );
	}
	QString dd ;
	QDate d = fileInfo.lastModified().date();
	QTime t = fileInfo.lastModified().time();
	dd.sprintf("%02d/%02d/%02d %02d:%02d",d.year()%100,d.month(),d.day(),t.hour(),t.minute());
	setText(FileView::FileDate,dd);
}

FileItem::~FileItem()
{
}

const QFileInfo* FileItem::info()
{
	return &fileInfo;
}

QString FileItem::key( int column, bool ascending ) const
{
	if( (column == FileView::FileName) && fileInfo.isDir() ){ 
		if ( myZaurus  && isMain_Memory(fileInfo.filePath())  ){ 
			return "!!"+ text(column).lower();
		} else if ( myZaurus  && isSD_Card(fileInfo.filePath())  ){ 
			return "!#"+ text(column).lower();
		} else if (text(column) == "..") {
			if (ascending) {
				return "!";
			} else {
				return QChar(0xffffU);
			}
		} else { 
			return "!"+ text(column).lower();
		}
	} else {
		return text(column).lower();
	}
}

bool FileItem::isMain_Memory(const QString &fp)
{
	return (fp +"/" == QPEApplication::documentDir()) ;
}
bool FileItem::isSD_Card(const QString &fp)
{
	return (fp == "/mnt/card" || fp == "/var/mnt/card" || fp == "/usr/mnt.rom/card" );
}

bool FileItem::isCF_Card(const QString &fp)
{
	return (fp == "/mnt/cf" || fp == "/var/mnt/cf" || fp == "/usr/mnt.rom/cf" );
}

bool FileItem::availSD_Card()
{
	QFileInfo f(pathSD_Card()+"/NotAvailable");
	if (f.exists() && (f.size() == 0) && !f.isDir()) {
		return FALSE;
	}
	return TRUE;
}

bool FileItem::availCF_Card()
{
	QFileInfo f(pathCF_Card()+"/NotAvailable");
	if (f.exists() && (f.size() == 0) && !f.isDir()) {
		return FALSE;
	}
	return TRUE;
}
const QString FileItem::pathMain_Memory()
{
	QString s = QPEApplication::documentDir();
	return s.left(s.length()-1);
}
