#include <qlistview.h>
#include <qfileinfo.h>

class FileView : public QListView
{
	Q_OBJECT

public:
	FileView( const QString &path , const QString &nameFilter = "*", bool hidden = FALSE, bool myZau = FALSE, bool dispF = TRUE, QWidget * parent = 0, const char * name = 0 , WFlags f = 0 );
	~FileView() ;
	const QFileInfo* selected();
	void updateView();
	void setCurrentFile(const QString &file);
	void setHidden(bool b);
	void setMyZaurus(bool b);
	bool getMyZaurus();
	void setNameFilter(const QString &filter);
	const QString currentDirName();
	const QString currentPathName();
	bool delFile (const QString &mymsg , bool forceF = FALSE) ;

	enum ColumnName {
		FileName,
		FileSize,
		FileDate
	};

signals:
	void fileSelected(const QFileInfo &);
	void selectionChanged(const QFileInfo &);
	void dirChanged();

protected:
	virtual void keyPressEvent ( QKeyEvent *e );
	virtual void resizeEvent(QResizeEvent *e);


private slots:
	void selectItem(QListViewItem *);
	void selectChanged(QListViewItem *);

private:
	bool getDirInfo(const QString &path);  // if err return true
	void changeDir(const QString &fp);
	void setColumnSize();
	bool removeFile(const QString &fp,bool isDir);

	QString currentPath;
	QString nFilter;
	bool hiddenFile;
	bool myZaurus;
	bool vga;
	bool dispFile;

};

class FileItem : public QListViewItem
{
//	Q_OBJECT

public:
	FileItem( QListView * parent ,const QFileInfo itemInfo);
	~FileItem();
	const QFileInfo *info();
	virtual QString key( int column, bool ascending = TRUE ) const;

	static bool isMain_Memory(const QString &fp);
	static bool isSD_Card(const QString &fp);
	static bool isCF_Card(const QString &fp);
	static bool availSD_Card();
	static bool availCF_Card();
	static inline const QString pathMyZaurus() { return "My Zaurus";};
	static const QString pathMain_Memory();
	static inline const QString pathSD_Card() { return "/mnt/card"; };
	static inline const QString pathCF_Card() { return "/mnt/cf"; };
	static inline const QString dirMain_Memory() { return "Documents"; };
	static inline const QString dirSD_Card() { return "card"; };
	static inline const QString dirCF_Card() { return "cf"; };

private:
	QFileInfo fileInfo;
	bool myZaurus;
};
