#include <qapplication.h>
#include <qobject.h>
#include <sys/stat.h>
#include <qfile.h>
#include <qtextcodec.h>

#include "MiscLib.h"

bool MiscLib::isVga()
{
	if (QApplication::desktop()->width() > HalfVgaWidth ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

const QString MiscLib::makeProperty(const QFileInfo &file,const QString &fileType)
{
	QString pr("");
	pr +=file.fileName() + "\n";
	if (fileType.length() >0 ) {
		pr +=QObject::tr("Type/Code : ") +fileType + "\n";
	}
	pr +=QObject::tr("File Path : ") +file.dirPath() + "\n";
	if (file.isSymLink() ) {
		pr += "         -> " + file.readLink() + "\n";
	}
	pr +=QObject::tr("Size      : ") +QString("%1").arg(file.size()) + "\n";
	pr +=QObject::tr("Last Mod  : ") +file.lastModified().toString() + "\n";
	pr +=QObject::tr("Owner     : ") +file.owner() + "\n";
	pr +=QObject::tr("Group     : ") +file.group() + "\n";
	pr +=QObject::tr("Permission: ") +"\n";
	QString p("");
	if (file.permission(QFileInfo::ReadUser))
		p+=QObject::tr("Read   ");
	else 
		p+=   "       ";
	if (file.permission(QFileInfo::WriteUser))
		p+=QObject::tr("Write  ");
	else 
		p+=   "       ";
	if (file.permission(QFileInfo::ExeUser))
		p+=QObject::tr("Exec   ");
	else 
		p+=   "       ";
	pr +=QObject::tr("  Owner   : ") +p + "\n";
	p = "";
	if (file.permission(QFileInfo::ReadGroup))
		p+=QObject::tr("Read   ");
	else 
		p+=   "       ";
	if (file.permission(QFileInfo::WriteGroup))
		p+=QObject::tr("Write  ");
	else 
		p+=   "       ";
	if (file.permission(QFileInfo::ExeGroup))
		p+=QObject::tr("Exec   ");
	else 
		p+=   "       ";
	pr +=QObject::tr("  Group   : ") +p + "\n";
	p = "";
	if (file.permission(QFileInfo::ReadOther))
		p+=QObject::tr("Read   ");
	else 
		p+=   "       ";
	if (file.permission(QFileInfo::WriteOther))
		p+=QObject::tr("Write  ");
	else 
		p+=   "       ";
	if (file.permission(QFileInfo::ExeOther))
		p+=QObject::tr("Exec   ");
	else 
		p+=   "       ";
	pr +=QObject::tr("  Other   : ") +p + "\n";
	return pr;
}

bool MiscLib::fileCopy( const QString & dest, const QString & src )
{
	char bf[ 50000 ];
	int  bytesRead;
	bool success = TRUE;
	struct stat status;

qDebug("dest %s",(const char *)dest);
qDebug("src  %s",(const char *)src);
	QFile s( src );
	QFile d( dest );

	if( s.open( IO_ReadOnly | IO_Raw ) &&
		d.open( IO_WriteOnly | IO_Raw ) )
	{
		while( (bytesRead = s.readBlock( bf, sizeof( bf ) )) ==
			   sizeof( bf ) )
		{
			if( d.writeBlock( bf, sizeof( bf ) ) != sizeof( bf ) ){
				success = FALSE;
				break;
			}
		}
		if( success && (bytesRead > 0) ){
			d.writeBlock( bf, bytesRead );
		}
	} else {
		success = FALSE;
	}
	s.close();
	d.close();

	// Set file permissions
	if( success && stat( src.utf8(), &status ) == 0 ){
		chmod( dest.utf8(), status.st_mode );
	}

	return success;
}

/*
    this function is 'checkcodec' from ZEditor v 1.3.1b
      2003.10.12 modified by Masakazu Ikezawa
*/


QString MiscLib::setCode(const char *bf,uint fsize)  // 2003.10.12
{
	uint		i;
	uint		c_count;
	uint		asc_count;		// v0.9.2
	uint		jis_flg;

	const unsigned char* buf =(const unsigned char*)bf;

	if (( buf[0] == 0xff ) && ( buf[1] == 0xfe ))
		return( "utf16" );						// 2003.10.12

	// ---------------------------------- check UTF-8
	for( i= 0, c_count= 0, asc_count= 0, jis_flg= 0 ; i < fsize ; i++ )
	{
		if ( c_count > 0 )
		{
			if (( buf[i] < 0x80 ) || ( buf[i] > 0xbf ))
				break;
			c_count--;
		}
		else	// c_count == 0
		{
			if (( buf[i] > 0x1b ) && ( buf[i] <= 0x7f ))
				asc_count++;

			if ( buf[i] == 0x1b )
				jis_flg++;

			if ( buf[i] <= 0x7f )   // 2003.10.14 for suppress warning
			{
				c_count = 0;
			}
			else
			{
				if (( buf[i] >= 0xc0 ) && ( buf[i] <= 0xdf ))
					c_count = 1;
				else
					if (( buf[i] >= 0xe0 ) && ( buf[i] <= 0xef ))
						c_count = 2;
					else
						break;
			}
		}
	}

	if (( i >= fsize ) && ( ! jis_flg ))
	{
		if ( asc_count == fsize )
			return( "SJIS" );		// default codec v0.9.2				// 2003.10.12
		else
			return( "utf8" );											// 2003.10.12
	}

	// ---------------------------------- check JIS( iso-2022-jp )
	for( i= 0, c_count= 0 ; i < fsize ; i++ )
	{
		// ------------------------------ v1.3.0b
		if ( c_count == 0 )
		{
			if ( buf[i] == 0x1b )
				c_count = 3;
		}
		else
		{
			if ( c_count == 3 )
			{
				if ( buf[i] == 0x24 )
					c_count = 2;
				else
					if ( buf[i] == 0x28 )
						c_count = 1;
					else
						c_count = 0;
			}
			else
			{
				if ( c_count == 2 )
				{
					if (( buf[i] == 0x40 ) || ( buf[i] == 0x42 ))
						return( "JIS7" );					// 2003.10.12
				}
				else
				{
					if ( c_count == 1 )
					{
						if (( buf[i] == 0x42 ) || ( buf[i] == 0x49 ) || ( buf[i] == 0x4a ))
							return( "JIS7" );				// 2003.10.12
					}
				}
				c_count = 0;
			}
		}
	}

	// ---------------------------------- check ShiftJIS
	for( i= 0, c_count= 0 ; i < fsize ; i++ )
	{
		if (( buf[i] >= 0x81 ) && ( buf[i] <= 0x9f )
		 && ( buf[i] != 0x8e ) && ( buf[i] != 0x9f ))
			return( "SJIS" );								// 2003.10.12
	}
	return( "eucJP");										// 2003.10.12
}
